/************************************************************************/
/*                                                                      */
/*      Zoom Page  -  Firefox Extension  -  Main Window                 */
/*                                                                      */
/*      Javascript for Main Window overlay                              */
/*                                                                      */
/*      Copyright (C) 2009-2017  by  DW-dev                             */
/*                                                                      */
/*      Last Edit  -  15 Aug 2017                                       */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Refer to Firefox source code: mozilla-release\...                    */
/*                                                                      */
/*   browser\base\content\browser-fullZoom.js - FullZoom                */
/*   toolkit\content\viewZoomOverlay.js       - ZoomManager             */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* FullZoom                                                             */
/*                                                                      */
/*   FullZoom.onContentPrefSet() - apply or remove site-specific        */
/*                               - redefined for fit-to-width           */
/*                                                                      */
/*   FullZoom.onContentPrefRemoved() - apply or remove site-specific    */
/*                                   - redefined for fit-to-width       */
/*                                                                      */
/*   FullZoom.onLocationChange() - apply or remove site-specific        */
/*                               - redefined for image zoom and         */
/*                                 the four different zoom modes        */
/*                                                                      */
/*   FullZoom.enlarge() - increase zoom level and apply site-specific   */
/*                      - redefined to set percent                      */
/*                                                                      */
/*   FullZoom.reduce()  - reduce zoom level and apply site-specific     */
/*                      - redefined to set percent                      */
/*                                                                      */
/*   FullZoom.reset()   - reset zoom level and remove site-specific     */
/*                      - redefined for image zoom and fit-to-width     */
/*                                                                      */
/*   _applyPrefToZoom()    - apply content preference to zoom value     */
/*   _applyPrefToSetting() - redefined for site-specific zoom type      */
/*                                                                      */
/*   _applyZoomToPref()    - apply zoom value to content preference     */
/*   _applySettingToPref() - redefined for site-specific zoom type      */
/*                                                                      */
/*   _removePref()      - remove content preference (make undefined)    */
/*                      - redefined for site-specific zoom type         */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* ZoomManager                                                          */
/*                                                                      */
/*   ZoomManager.MIN           - minimum zoom value                     */
/*                                                                      */
/*   ZoomManager.MAX           - maximum zoom value                     */
/*                                                                      */
/*   ZoomManager.zoomValues[]  - array of zoom values                   */
/*                                                                      */
/*   ZoomManager.snap()        - snap to closest zoom value             */
/*                                                                      */
/*   ZoomManager.useFullZoom   - use full-page zoom                     */
/*                                                                      */
/*   ZoomManager.zoom          - zoom value for selected browser        */
/*                                                                      */
/*   ZoomManager.getZoomForBrowser() - get zoom value                   */
/*                                   - redefined for image zoom         */
/*                                                                      */
/*   ZoomManager.setZoomForBrowser() - set zoom value                   */
/*                                   - redefined for image zoom and     */
/*                                     text minimum zoom level          */
/*                                                                      */
/*   ZoomManager.enlarge()     - increase zoom level                    */
/*                             - redefined to avoid using snap()        */
/*                             - implicitly calls setZoomForBrowser()   */
/*                                                                      */
/*   ZoomManager.reduce()      - reduce zoom level                      */
/*                             - redefined to avoid using snap()        */
/*                             - implicitly calls setZoomForBrowser()   */
/*                                                                      */
/*   ZoomManager.reset()       - reset zoom level to 1                  */
/*                             - implicitly calls setZoomForBrowser()   */
/*                                                                      */
/*   ZoomManager.toggleZoom()  - toggle zoom (text-only or full-page)   */
/*                             - redefined for site-specific zoom type  */
/*                               and fit-to-width zoom                  */
/*                             - implicitly calls setZoomForBrowser()   */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Zoom Modes                                                           */
/*                                                                      */
/*   Default Zoom         -  Applied when tab opened.                   */
/*   (D/X)                                                              */
/*                                                                      */
/*   Fit-To-Width Zoom    -  Applied when different page loaded,        */
/*   (F/X)                   or when page manually reloaded,            */
/*                           or when browser window resized.            */
/*                                                                      */
/*   Default Zoom +       -  Applied when page loaded or reloaded,      */
/*   Site-Specific Zoom      or when page manually reloaded,            */
/*   (D/S)                   or when tab selected. If site value not    */
/*                           defined, then set to Default Zoom.         */
/*                                                                      */
/*   Fit-To-Width Zoom +  -  Applied when page loaded or reloaded,      */
/*   Site-Specific Zoom      or when page manually reloaded,            */
/*   (F/S)                   or when tab selected. If site value not    */
/*                           defined, then set to Fit-To-Width Zoom.    */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Display Indicators                                                   */
/*                                                                      */
/*   X - Explict Zoom applied.                                          */
/*   D - Default Zoom applied.                                          */
/*   F - Fit-To-Width/Window Zoom applied.                              */
/*   S - Site-Specific Zoom applied.                                    */
/*   P - Applied to Full-Page.                                          */
/*   T - Applied to Text-Only.                                          */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Event Actions                                                        */
/*                                                                      */
/*   TabOpen            - Set default zoom.                             */
/*                                                                      */
/*   ContentPrefSet     - Set last fit-to-width or site-specific zoom.  */
/*                                                                      */
/*   ContentPrefRemoved - Set last fit-to-width or site-specific zoom.  */
/*                                                                      */
/*   LocationChange     - Set last fit-to-width or site-specific zoom.  */
/*                                                                      */
/*   DOMTitleChanged    - Set image default or fit-to-window zoom.      */
/*                                                                      */
/*   DOMContentLoaded   - Not used.                                     */
/*                                                                      */
/*   load               - Not used.                                     */
/*                                                                      */
/*   pageshow           - Set fit-to-width zoom.                        */
/*                      - Set image default or fit-to-window zoom.      */
/*                                                                      */
/*   hashchange         - Set fit-to-width zoom.                        */
/*                                                                      */
/*   TabSelect          - Set fit-to-width zoom.                        */
/*                                                                      */
/*   overflow           - Not used.                                     */
/*                                                                      */
/*   underflow          - Not used.                                     */
/*                                                                      */
/*   resize             - Set fit-to-width zoom.                        */
/*                      - Set image fit-to-window zoom.                 */
/*                                                                      */
/*                                                                      */
/* Event Notes                                                          */
/*                                                                      */
/*   DOMContentLoaded and load events are not fired if page is          */
/*   retrieved from backward/forward cache.                             */
/*                                                                      */
/*   The order of LocationChange, DOMContentLoaded, load and            */
/*   pageshow events can vary.                                          */
/*                                                                      */
/*   The order of LocationChange and TabSelect events can vary.         */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/* Firefox Scaling Options (about:config)                               */
/*                                                                      */
/*   browser.enable_automatic_image_resizing                            */
/*                                                                      */
/*     false - similar to setting 'Image Zoom Mode' to 'Default'        */
/*     true  - similar to setting 'Image Zoom Mode' to 'Fit-To-Window'  */
/*                                                                      */
/*   browser.enable_click_image_resizing                                */
/*                                                                      */
/*     false - same as disabling 'Click on image to apply ...'          */
/*     true  - same as enabling 'Click on image to apply ...'           */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Additional Browser State Properties                                  */
/*                                                                      */
/*   zoompage_zoomType                                                  */
/*   zoompage_ftwType                                                   */
/*                                                                      */
/*   zoompage_normalZoom                                                */
/*   zoompage_imageZoom                                                 */
/*                                                                      */
/*   zoompage_previousURI                                               */
/*   zoompage_previousWidth                                             */
/*   zoompage_previousHeight                                            */
/*                                                                      */
/*   zoompage_pageState                                                 */
/*   zoompage_notZoomed                                                 */
/*                                                                      */
/*   zoompage_lastFitToWidth                                            */
/*   zoompage_lastFitToWindow                                           */
/*                                                                      */
/*   zoompage_imageWidth                                                */
/*   zoompage_imageHeight                                               */
/*                                                                      */
/*   zoompage_errorShown                                                */
/*   zoompage_imageShown                                                */
/*   zoompage_specialShown                                              */
/*                                                                      */
/*   zoompage_resizeTimeout                                             */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Notes on Compatibility with Multi-Process Firefox [e10s]             */
/*                                                                      */
/* 1. <multiprocessCompatible> property is not set in install.rdf.      */
/*                                                                      */
/* 2. Access to the following content DOM data in e10s windows          */
/*    relies on automatically generated CPOW's:                         */
/*                                                                      */
/*      - it is believed that all CPOW's have been eliminated           */
/*                                                                      */
/* 3. A content frame script is loaded into all browsers to:            */
/*                                                                      */
/*      - listen for DOMTitleChanged, pageshow, hashchange, resize,     */
/*        DOMMouseScroll and click events                               */
/*      - perform title change, mouse scroll and content click          */
/*        functions for image pages                                     */
/*      - perform set image cursor, set image background, set image     */
/*        zoom and fit-to-width zoom functions                          */
/*                                                                      */
/* 4. All "unsafe CPOW usage" warnings due to add-on code have been     */
/*    avoided by moving functionality into the content frame script.    */
/*                                                                      */
/************************************************************************/

"use strict";

var zoomPage =
{
    /********************************************************************/
    
    /* Shared variables */
    
    appInfo: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
    versionComparator: Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator),
    ffVersion: "",
    ffPaleMoon: false,
    
    runtime: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime),
    osPlatform: "",
    
    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.zoompage."),
    
    otherPrefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch),
    
    contentPrefs: null,
    
    promptService: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),

    winmed: Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator),
    
    forcedRestart: false,
    
    ignorePrefChanged: false,
    
    zoomLevels: new Array("30-50-67-80-90-100-110-120-133-150-170-200-240-300",
                          "30-50-75-100-125-150-200-300-400",
                          "30-50-75-90-100-110-125-150-200-300-400",
                          "30-50-67-80-90-100-110-120-133-150-170-200-240-300-400",
                          "25-50-75-100-125-150-200-300-400-500",
                          "25-50-75-90-100-110-125-150-200-300-400-500",
                          "25-50-67-80-90-100-110-120-133-150-170-200-240-300-400-500"),
    
    levelArray: new Array(),
    
    zoomMode: 2,
    imagezoomMode: 1,
    
    lastFitToWidth: 1.0,
    
    button: 0,
    shiftKey: false,
    ctrlKey: false,

    modestatusInterval: null,
    modestatusCount: 0,
    
    ftwCallbackRef: new Array(),
    ftwCallbackFunc: new Array(),
    
    traceEnable: false,
    
    historyEnable: false,
    historyCount: 0,
    historyString: "",
    
    debugCount: 0,
    
    /********************************************************************/
    
    /* Listen for changes to settings */
    
    prefsObserver:
    {
        register: function()
        {
            /* Query nsIPrefBranch2 interface to observe changes to pref values */
            zoomPage.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
            
            /* Add the observer */
            zoomPage.prefs.addObserver("",this,false);
        },
        
        observe: function(subject,topic,data)
        {
            var i,lighttext,indicator,values,browser;
            var colors = new Array();
            
            if (topic != "nsPref:changed") return;
            
            if (zoomPage.ignorePrefChanged) { zoomPage.ignorePrefChanged = false; return; }
            
            /* Load option settings */
            
            zoomPage.zoomMode = zoomPage.prefs.getIntPref("zoommode");
            zoomPage.imagezoomMode = zoomPage.prefs.getIntPref("imagezoommode");
            
            /* Button style */
            
            document.getElementById("main-window").setAttribute("zoompage-buttonstyle",zoomPage.prefs.getCharPref("buttonstyle"));
            
            /* Automatic button icon/border colors */
            
            colors =  window.getComputedStyle(document.getElementById("main-window"),null).
                      getPropertyValue("color").match(/rgb\((\d+),\s(\d+),\s(\d+)\)/);
            
            lighttext = (Number(colors[1])+Number(colors[2])+Number(colors[3]) >= 0x80+0x80+0x80);  /* light text */
            
            document.getElementById("main-window").setAttribute("zoompage-lighticons",lighttext);
            document.getElementById("main-window").setAttribute("zoompage-darkborders",lighttext);
            
            /* Adjust button icon/border colors */
            
            if (zoomPage.prefs.getIntPref("buttoncolors") == 1) document.getElementById("main-window").setAttribute("zoompage-lighticons",!lighttext);
            else if (zoomPage.prefs.getIntPref("buttoncolors") == 2) document.getElementById("main-window").setAttribute("zoompage-darkborders",!lighttext);
            else if (zoomPage.prefs.getIntPref("buttoncolors") == 3)
            {
                document.getElementById("main-window").setAttribute("zoompage-lighticons",!lighttext);
                document.getElementById("main-window").setAttribute("zoompage-darkborders",!lighttext);
            }
            
            /* Firefox Zoom Indicator */
            
            indicator = document.getElementById("urlbar-zoom-button");
            
            if (indicator != null) indicator.setAttribute("showindicator",zoomPage.prefs.getBoolPref("showindicator"));  /* FF51+ */
            
            /* Zoom Levels */
            
            zoomPage.zoomLevels[7] = zoomPage.prefs.getCharPref("customlevels");
            
            zoomPage.levelArray = zoomPage.zoomLevels[zoomPage.prefs.getIntPref("zoomlevels")].split("-");
            
            values = "" + zoomPage.levelArray[0]/100;
            for (i = 1; i < zoomPage.levelArray.length; i++) values = values + "," + zoomPage.levelArray[i]/100;
            
            zoomPage.otherPrefs.setIntPref("zoom.minPercent",zoomPage.levelArray[0]);
            zoomPage.otherPrefs.setIntPref("zoom.maxPercent",zoomPage.levelArray[zoomPage.levelArray.length-1]);
            zoomPage.otherPrefs.setCharPref("toolkit.zoomManager.zoomValues",values);
            
            delete ZoomManager.MIN;
            ZoomManager.MIN = zoomPage.levelArray[0]/100;
            delete ZoomManager.MAX;
            ZoomManager.MAX = zoomPage.levelArray[zoomPage.levelArray.length-1]/100;
            ZoomManager.zoomValues.length = 0;
            for (i = 0; i < zoomPage.levelArray.length; i++) ZoomManager.zoomValues[i] = zoomPage.levelArray[i]/100;
            
            /* Synchronize Firefox's Site-Specific zoom preference */
            
            zoomPage.otherPrefs.setBoolPref("browser.zoom.siteSpecific",zoomPage.zoomMode >> 1);
            
            /* Initialize previous width/height for all unselected tabs - to update zoom level when selected */
            
            for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
            {
                browser = gBrowser.tabContainer.childNodes[i].linkedBrowser;
                
                if (browser != gBrowser.selectedBrowser)
                {
                    browser.zoompage_previousWidth = 0;
                    browser.zoompage_previousHeight = 0;
                }
            }
            
            /* Update zoom level for selected tab */
            
            browser = gBrowser.selectedBrowser;
            
            if (zoomPage.imagePage(browser))  /* image page */
            {
                /* Set styles for image background appearance */
                
                zoomPage.setImageBackground(browser,true);
                
                /* Adjust fit-to-window type for selected tab */
                
                browser.zoompage_ftwType = zoomPage.prefs.getIntPref("imagedefftwtype");
                
                /* Adjust zoom value for selected tab - in case zoom levels have been changed */
                
                if (ZoomManager.zoom < 1/100) ZoomManager.zoom = 1/100;
                else if (ZoomManager.zoom > 10000/100) ZoomManager.zoom = 10000/100;
                
                /* Default Zoom - in case zoom mode has been changed */
                
                if (zoomPage.imagezoomMode == 0)
                {
                    ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
                }
                
                /* Fit-To-Window Zoom - in case zoom mode has been changed */
                
                if (zoomPage.imagezoomMode == 1)
                {
                    zoomPage.fittowindowZoom(browser,false);
                }
                
                /* Update zoom percentage and indicators */
                
                zoomPage.setPercent(1);
            }
            else  /* not image page */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    var doftw;
                    
                    doftw = false;
                    
                    /* Adjust zoom type for selected tab */
                    
                    ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                    browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    
                    /* Adjust zoom value for selected tab - in case zoom levels have been changed */
                    
                    if (ZoomManager.zoom < ZoomManager.MIN) ZoomManager.zoom = ZoomManager.MIN;
                    else if (ZoomManager.zoom > ZoomManager.MAX) ZoomManager.zoom = ZoomManager.MAX;
                    
                    /* Site-Specific Zoom - apply saved site-specific zoom value */
                    
                    if ((zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3) && typeof sitevalue != "undefined")
                    {
                        if (sitevalue < ZoomManager.MIN) ZoomManager.zoom = ZoomManager.MIN;
                        else if (sitevalue > ZoomManager.MAX) ZoomManager.zoom = ZoomManager.MAX;
                        else ZoomManager.zoom = sitevalue;
                    }
                    
                    /* Default Zoom or Default Zoom + Site-Specific Zoom (if site-specific not set) - in case zoom mode has been changed */
                    
                    if (zoomPage.zoomMode == 0 || (zoomPage.zoomMode == 2 && typeof sitevalue == "undefined"))
                    {
                        zoomPage.defaultZoom(browser);
                    }
                    
                    /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - in case zoom mode has been changed */
                    
                    if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                    {
                        doftw = true;
                    }
                    
                    zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"pob",
                    function(browser,sitevalue)
                    { 
                        /* Site-Specific Zoom - update saved site-specific value */
                        
                        if (zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3)
                        {
                            if (typeof sitevalue != "undefined")
                            {
                                if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                                else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                                else FullZoom._applySettingToPref();
                            }
                        }
                        
                        /* Synchronize Firefox's Site-Specific Zoom global zoom value content preference */
                        
                        if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                        {
                            zoomPage.setGlobalSiteSpecific(FullZoom.name,browser.zoompage_lastFitToWidth);  /* set global zoom value content preference same as fit-to-width zoom */
                        }
                        else
                        {
                            zoomPage.setGlobalSiteSpecific(FullZoom.name,zoomPage.defaultLevel(browser)/100);  /* set global zoom value content preference same as default zoom */
                        }
                        
                        /* Update zoom percentage and indicators */
                        
                        zoomPage.setPercent(2);
                    });
                });
            }
        }
    },
    
    /********************************************************************/
    
    /* Listen for changes to toolbar button location (FF29+) */
    
    widgetListener:
    {
        onWidgetAdded: function(aWidgetId,aArea,aPosition)
        {
            zoomPage.setPercent(3);
        },
        
        onCustomizeStart: function(aWindow)
        {
            window.setTimeout(
            function()
            {
                var percent;
                
                if (document.getElementById("zoompage-item") != null)  /* button in Customize tab - in toolbar, menu panel or palette */
                {
                    /* if button in toolbar */
                    
                    percent = document.getElementById("zoompage-percent-label-" + zoomPage.activeButtonStyle());
                    
                    percent.setAttribute("value","100%");
                    percent.style.removeProperty("font-style");
                    percent.style.removeProperty("color");
                    percent.style.removeProperty("outline-width");
                    percent.style.removeProperty("outline-style");
                    percent.style.removeProperty("outline-offset");
                    
                    /* if button in menu panel or palette */
                    
                    percent = document.getElementById("zoompage-percent-label-n");
                    
                    percent.setAttribute("value","100%");
                    percent.style.removeProperty("font-style");
                    percent.style.removeProperty("color");
                    percent.style.removeProperty("outline-width");
                    percent.style.removeProperty("outline-style");
                    percent.style.removeProperty("outline-offset");
                }
            },150);
        }
    },
    
    /********************************************************************/
    
    /* Load and Unload functions */
    
    onLoad: function()
    {
        var i,navbar,lighttext,indicator,values,percent,zoomDisplay,menuPopup,windowMM;
        var defaultlevel,fittowidthmin,fittowidthmax,fittowidthful,imgdefaultlevel,imgfittowindowmin,imgfittowindowmax;
        var colors = new Array();
        
        window.removeEventListener("load",zoomPage.onLoad,false);

        /* Determine Firefox version and set attribute */
        
        if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"34.0a1") >= 0) zoomPage.ffVersion = "34.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"30.0a1") >= 0) zoomPage.ffVersion = "30.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"29.0a1") >= 0) zoomPage.ffVersion = "29.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"27.0a1") >= 0) zoomPage.ffVersion = "27.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"25.0a1") >= 0) zoomPage.ffVersion = "25.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"24.0a1") >= 0) zoomPage.ffVersion = "24.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"22.0a1") >= 0) zoomPage.ffVersion = "22.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"19.0a1") >= 0) zoomPage.ffVersion = "19.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"17.0a1") >= 0) zoomPage.ffVersion = "17.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"14.0a1") >= 0) zoomPage.ffVersion = "14.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"12.0a1") >= 0) zoomPage.ffVersion = "12.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"11.0a1") >= 0) zoomPage.ffVersion = "11.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"9.0a1") >= 0) zoomPage.ffVersion = "9.0";
        else zoomPage.ffVersion = "4.0";
        
        document.getElementById("main-window").setAttribute("zoompage-ffversion",zoomPage.ffVersion);
        
        /* Determine operating system platform and set attribute */
        
        zoomPage.osPlatform = zoomPage.runtime.OS;
        
        document.getElementById("main-window").setAttribute("zoompage-osplatform",zoomPage.osPlatform);
        
        /* Determine if Pale Moon browser */
        
        if (zoomPage.appInfo.name == "Pale Moon") zoomPage.ffPaleMoon = true;
        
        /* Determine if forced restart */
        
        zoomPage.forcedRestart = zoomPage.otherPrefs.getBoolPref("browser.sessionstore.resume_session_once");
        
        /* Check whether Old Default Image Style add-on installed */
        
        Components.utils.import("resource://gre/modules/AddonManager.jsm");
        AddonManager.getAddonByID("olddefaultimagestyle@dagger2-addons.mozilla.org",
        function(addon)
        {
            var title,message,checkmsg;
            var check = { value: false }; 
            
            if (addon && !addon.userDisabled)
            {
                check.value = zoomPage.prefs.getBoolPref("odisdonotshow");
                
                if (!check.value)
                {
                    title = document.getElementById("zoompage-prompttitle").getAttribute("value");
                    message = document.getElementById("zoompage-odiswarning").getAttribute("value");
                    checkmsg = document.getElementById("zoompage-odisdonotshow").getAttribute("value");
                    
                    zoomPage.promptService.alertCheck(null,title,message,checkmsg,check);
                    
                    zoomPage.prefs.setBoolPref("odisdonotshow",check.value);
                }
            }
        });
        
        /* Initialize content preferences reference */
        
        if (zoomPage.ffVersion >= +"22.0") zoomPage.contentPrefs = Components.classes["@mozilla.org/content-pref/service;1"].getService(Components.interfaces.nsIContentPrefService2);
        else zoomPage.contentPrefs = Components.classes["@mozilla.org/content-pref/service;1"].getService(Components.interfaces.nsIContentPrefService);
        
        /* Migrate option settings to Version 5.0 */
        
        if (!zoomPage.prefs.getBoolPref("migrated50"))
        {
            zoomPage.prefs.setBoolPref("swapactions",zoomPage.prefs.getBoolPref("leftclick"));
            zoomPage.prefs.setIntPref("defaultlevel",zoomPage.prefs.getIntPref("defaultzoom"));
            
            zoomPage.prefs.setBoolPref("migrated50",true);
        }
        
        /* Migrate option settings to Version 6.0 */
        
        if (!zoomPage.prefs.getBoolPref("migrated60"))
        {
            zoomPage.prefs.setIntPref("automaticmode",zoomPage.prefs.getIntPref("automaticzoom"));
            
            zoomPage.prefs.setBoolPref("migrated60",true);
        }
        
        /* Migrate option settings to Version 7.0 */
        
        if (!zoomPage.prefs.getBoolPref("migrated70"))
        {
            zoomPage.prefs.setIntPref("zoommode",zoomPage.prefs.getIntPref("automaticmode"));
            zoomPage.prefs.setBoolPref("toprefix",zoomPage.prefs.getBoolPref("prefixpercent"));
            zoomPage.prefs.setBoolPref("toitalic",zoomPage.prefs.getBoolPref("italicpercent"));
            zoomPage.prefs.setBoolPref("tored",zoomPage.prefs.getBoolPref("redpercent"));
            
            zoomPage.prefs.setBoolPref("migrated70",true);
        }
        
        /* Migrate option settings to Version 8.0 */
        
        if (!zoomPage.prefs.getBoolPref("migrated80"))
        {
            zoomPage.prefs.setIntPref("imagezoommode",1);
            zoomPage.prefs.setIntPref("imgdefaultlevel",100);
            
            if (zoomPage.prefs.prefHasUserValue("imagelevel"))
            {
                if (zoomPage.prefs.getIntPref("imagelevel") <= 1) zoomPage.prefs.setIntPref("imagezoommode",0);
                if (zoomPage.prefs.getIntPref("imagelevel") == 1) zoomPage.prefs.setIntPref("imgdefaultlevel",zoomPage.prefs.getIntPref("defaultlevel"));
            }
            
            zoomPage.prefs.setBoolPref("migrated80",true);
        }
        
        /* Migrate option settings to Version 10.0 */
        
        if (!zoomPage.prefs.getBoolPref("migrated100"))
        {
            zoomPage.prefs.setCharPref("editcustomlevels",zoomPage.prefs.getCharPref("customlevels"));
            zoomPage.prefs.setIntPref("imgfittowindowmin",1);
            zoomPage.prefs.setIntPref("imgfittowindowmax",300);
            
            zoomPage.prefs.setBoolPref("migrated100",true);
        }
        
        /* Migrate option settings to Version 13.2 */
        
        if (!zoomPage.prefs.getBoolPref("migrated132"))
        {
            zoomPage.prefs.setBoolPref("specialenable",false);
            
            zoomPage.prefs.setBoolPref("migrated132",true);
        }
        
        /* Migrate option settings to Version 14.3 */
        
        if (!zoomPage.prefs.getBoolPref("migrated143"))
        {
            zoomPage.prefs.setBoolPref("swapmouseclicks",zoomPage.prefs.getBoolPref("swapactions"));
            
            zoomPage.prefs.setBoolPref("migrated143",true);
        }
        
        /* Load option settings */
        
        zoomPage.zoomMode = zoomPage.prefs.getIntPref("zoommode");
        zoomPage.imagezoomMode = zoomPage.prefs.getIntPref("imagezoommode");
        
        /* Redefine existing FullZoom functions (see browser-fullZoom.js) */
        
        zoomPage.redefineFullZoomFunctions();
        
        /* Redefine existing ZoomManager functions (see viewZoomOverlay.js) */
        
        zoomPage.redefineZoomManagerFunctions();
        
        /* On first run after installation */
        
        if (zoomPage.prefs.getBoolPref("firstrun"))
        {
            /* Display accessibility message */
            
            window.openDialog("chrome://zoompage/content/zoompage-install.xul","","chrome,dialog,titlebar,centerscreen,alwaysRaised");
            
            /* Add button to navigation toolbar */
            
            if (zoomPage.ffVersion >= +"29.0")
            {
                if (CustomizableUI.getPlacementOfWidget("zoompage-item") == null)  /* button in palette */
                {
                    CustomizableUI.addWidgetToArea("zoompage-item","nav-bar",null);
                }
            }
            else
            {
                if (document.getElementById("zoompage-item") == null)  /* button in palette */
                {
                    navbar = document.getElementById("nav-bar");
                    navbar.setAttribute("collapsed","false");
                    navbar.insertItem("zoompage-item",null,null,false);
                    navbar.setAttribute("currentset",navbar.currentSet);
                    document.persist("nav-bar","currentset");
                }
            }
            
            zoomPage.prefs.setBoolPref("firstrun",false);
        }
        
        /* Button style */
        
        document.getElementById("main-window").setAttribute("zoompage-buttonstyle",zoomPage.prefs.getCharPref("buttonstyle"));
        
        /* Automatic button icon/border colors */
        
        colors =  window.getComputedStyle(document.getElementById("main-window"),null).
                  getPropertyValue("color").match(/rgb\((\d+),\s(\d+),\s(\d+)\)/);
        
        lighttext = (Number(colors[1])+Number(colors[2])+Number(colors[3]) >= 0x80+0x80+0x80);  /* light text */
        
        document.getElementById("main-window").setAttribute("zoompage-lighticons",lighttext);
        document.getElementById("main-window").setAttribute("zoompage-darkborders",lighttext);
        
        /* Adjust button icon/border colors */
        
        if (zoomPage.prefs.getIntPref("buttoncolors") == 1) document.getElementById("main-window").setAttribute("zoompage-lighticons",!lighttext);
        else if (zoomPage.prefs.getIntPref("buttoncolors") == 2) document.getElementById("main-window").setAttribute("zoompage-darkborders",!lighttext);
        else if (zoomPage.prefs.getIntPref("buttoncolors") == 3)
        {
            document.getElementById("main-window").setAttribute("zoompage-lighticons",!lighttext);
            document.getElementById("main-window").setAttribute("zoompage-darkborders",!lighttext);
        }
        
        /* Firefox Zoom Indicator */
        
        indicator = document.getElementById("urlbar-zoom-button");
        
        if (indicator != null) indicator.setAttribute("showindicator",zoomPage.prefs.getBoolPref("showindicator"));  /* FF51+ */
        
        /* Zoom Levels */
        
        zoomPage.zoomLevels[7] = zoomPage.prefs.getCharPref("customlevels");
        
        zoomPage.levelArray = zoomPage.zoomLevels[zoomPage.prefs.getIntPref("zoomlevels")].split("-");
        
        values = "" + zoomPage.levelArray[0]/100;
        for (i = 1; i < zoomPage.levelArray.length; i++) values = values + "," + zoomPage.levelArray[i]/100;
        
        zoomPage.otherPrefs.setIntPref("zoom.minPercent",zoomPage.levelArray[0]);
        zoomPage.otherPrefs.setIntPref("zoom.maxPercent",zoomPage.levelArray[zoomPage.levelArray.length-1]);
        zoomPage.otherPrefs.setCharPref("toolkit.zoomManager.zoomValues",values);
        
        delete ZoomManager.MIN;
        ZoomManager.MIN = zoomPage.levelArray[0]/100;
        delete ZoomManager.MAX;
        ZoomManager.MAX = zoomPage.levelArray[zoomPage.levelArray.length-1]/100;
        ZoomManager.zoomValues.length = 0;
        for (i = 0; i < zoomPage.levelArray.length; i++) ZoomManager.zoomValues[i] = zoomPage.levelArray[i]/100;
        
        /* Validate Default Zoom level preference */
        
        defaultlevel = zoomPage.prefs.getIntPref("defaultlevel");
        if (zoomPage.levelArray.indexOf(String(defaultlevel)) == -1) zoomPage.prefs.setIntPref("defaultlevel",100);
        defaultlevel = zoomPage.prefs.getIntPref("defaultlevel");
        
        /* Validate Fit-To-Width minimum and maximum zoom level preferences */
        
        fittowidthmin = zoomPage.prefs.getIntPref("fittowidthmin");
        if (fittowidthmin > defaultlevel || zoomPage.levelArray.indexOf(String(fittowidthmin)) == -1) zoomPage.prefs.setIntPref("fittowidthmin",defaultlevel);
        
        fittowidthmax = zoomPage.prefs.getIntPref("fittowidthmax");
        if (fittowidthmax < defaultlevel || zoomPage.levelArray.indexOf(String(fittowidthmax)) == -1) zoomPage.prefs.setIntPref("fittowidthmax",defaultlevel);
        fittowidthmax = zoomPage.prefs.getIntPref("fittowidthmax");
        
        fittowidthful = zoomPage.prefs.getIntPref("fittowidthful");
        if (fittowidthful < fittowidthmax || zoomPage.levelArray.indexOf(String(fittowidthful)) == -1) zoomPage.prefs.setIntPref("fittowidthful",fittowidthmax);
        
        /* Validate Image Default Zoom level preference */
        
        imgdefaultlevel = zoomPage.prefs.getIntPref("imgdefaultlevel");
        if (zoomPage.levelArray.indexOf(String(imgdefaultlevel)) == -1) zoomPage.prefs.setIntPref("imgdefaultlevel",100);
        imgdefaultlevel = zoomPage.prefs.getIntPref("imgdefaultlevel");
        
        /* Validate Image Fit-To-Window minimum and maximum zoom level preferences */
        
        imgfittowindowmin = zoomPage.prefs.getIntPref("imgfittowindowmin");
        if (imgfittowindowmin > imgdefaultlevel) zoomPage.prefs.setIntPref("imgfittowindowmin",imgdefaultlevel);
        
        imgfittowindowmax = zoomPage.prefs.getIntPref("imgfittowindowmax");
        if (imgfittowindowmax < imgdefaultlevel) zoomPage.prefs.setIntPref("imgfittowindowmax",imgdefaultlevel);
        
        /* Synchronize Firefox's Site-Specific Zoom preference */
        
        zoomPage.otherPrefs.setBoolPref("browser.zoom.siteSpecific",zoomPage.zoomMode >> 1);
        
        /* Synchronize Firefox's Site-Specific Zoom global zoom value content preference */
        
        delete FullZoom._globalValue;  /* too soon to call FullZoom.reset() */
        FullZoom._globalValue = zoomPage.prefs.getIntPref("defaultlevel")/100;  /* too soon to call FullZoom.reset() */
        
        zoomPage.setGlobalSiteSpecific(FullZoom.name,zoomPage.prefs.getIntPref("defaultlevel")/100);  /* set global zoom value content preference */
        
        /* Initialize Site-Specific Zoom global zoom type content preference */
        
        zoomPage.setGlobalSiteSpecific("browser.content.full-type",zoomPage.prefs.getIntPref("defaulttype"));  /* set global zoom type content preference */
        
        /* Disable Firefox's Image Resizing (scaling) preferences */
        
        zoomPage.otherPrefs.setBoolPref("browser.enable_automatic_image_resizing",false);
        zoomPage.otherPrefs.setBoolPref("browser.enable_click_image_resizing",false);
        
        /* Register preferences observer */
        
        zoomPage.prefsObserver.register();
        
        /* Add listeners for tabs */
        
        gBrowser.tabContainer.addEventListener("TabOpen",zoomPage.onTabOpen,false);
        gBrowser.tabContainer.addEventListener("TabSelect",zoomPage.onTabSelect,false);
        
        /* Add listeners for manual reloads and manual zoom page shortcuts */
        
        window.addEventListener("keypress",zoomPage.onWindowKeyPress,true);  /* reload shortcuts (F5 or Ctrl+R) and zoom page shortcuts */
        document.getElementById("context-reload").addEventListener("click",zoomPage.onReloadClick,false);  /* reload on content context menu */
        document.getElementById("context_reloadTab").addEventListener("click",zoomPage.onReloadClick,false);  /* reload tab on tab context menu */
        document.getElementById("context_reloadAllTabs").addEventListener("click",zoomPage.onReloadAllClick,false);  /* reload all tabs on tab context menu */
        if (document.getElementById("reload-button") != null) document.getElementById("reload-button").addEventListener("click",zoomPage.onReloadClick,false);  /* reload button */
        if (document.getElementById("urlbar-reload-button") != null) document.getElementById("urlbar-reload-button").addEventListener("click",zoomPage.onReloadClick,false);  /* urlbar reload button (FF29+) */
        
        /* Add listener for button added to toolbar */
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            CustomizableUI.addListener(zoomPage.widgetListener);
        }
        else
        {
            document.getElementById("navigator-toolbox").addEventListener("drop",zoomPage.onButtonDrop,false);
            document.getElementById("addon-bar").addEventListener("drop",zoomPage.onButtonDrop,false);
        }
        
        /* Add listener for Menu Panel or Overflow Panel showing */
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            document.getElementById("PanelUI-popup").addEventListener("popupshowing",zoomPage.onPanelUIPopupShowing,false);
            document.getElementById("widget-overflow").addEventListener("popupshowing",zoomPage.onOverflowPopupShowing,false);
        }
        
        /* Initialize display field width and zoom menu width (menuPopup.boxObject.width) */
        
        if (document.getElementById("zoompage-item") != null)  /* button accessible */
        {
            zoomPage.calculateDisplayWidth();
            
            percent = document.getElementById("zoompage-percent-label-" + zoomPage.activeButtonStyle());
            percent.setAttribute("value","");
            
            zoomDisplay = document.getElementById("zoompage-zoomdisplay-" + zoomPage.activeButtonStyle());
            
            menuPopup = zoomDisplay.childNodes[0];
            
            menuPopup.openPopup(zoomDisplay,"after_start",0,0,false,false,null);
            menuPopup.hidePopup();
        }
        
        /* Load frame script into all browsers in this window */
        
        windowMM = window.messageManager;
        windowMM.loadFrameScript("chrome://zoompage/content/zoompage-framescript.js",true);
        
        /* Add message listeners for all browsers */
        
        windowMM.addMessageListener("zoompage-ondomtitlechanged",zoomPage.onDOMTitleChanged);  /* (width,height) */
        windowMM.addMessageListener("zoompage-ondomcontentloaded",zoomPage.onDOMContentLoaded);
        windowMM.addMessageListener("zoompage-onpageshow",zoomPage.onPageShow);  /* (state) */
        windowMM.addMessageListener("zoompage-onhashchange",zoomPage.onHashChange);
        windowMM.addMessageListener("zoompage-onresize",zoomPage.onBrowserResize);
        windowMM.addMessageListener("zoompage-onscroll",zoomPage.onBrowserScroll);  /* (detail) */
        windowMM.addMessageListener("zoompage-onclick",zoomPage.onContentClick);  /* (clientx,clienty,scrollx,scrolly,scrollmaxx,scrollmaxy,ctrlkey) */
        windowMM.addMessageListener("zoompage-fittowidthzoom-reply",zoomPage.fittowidthZoom_Reply);  /* (sitevalue,reference) */
        windowMM.addMessageListener("zoompage-debugmessage",zoomPage.debugMessageFrameScript);  /* (module,information) */
        
        /* Allow time for initialization and for frame script to load into browsers */
        
        window.setTimeout(function() { zoomPage.onLoadDelayed(); },150);
    },
    
    onLoadDelayed: function()
    {
        var i,tab,browser,startupPage;
        var message = new Object();
        
        /* Do same as onTabOpen() for tabs that do not fire TabOpen events on browser startup */
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            tab = gBrowser.tabContainer.childNodes[i];
            
            browser = tab.linkedBrowser;
            
            /* Initialize page state */
            
            browser.zoompage_pageState = 0;  /* no page */
            
            /* Initialize normal/image zoom values*/
            
            browser.zoompage_normalZoom = zoomPage.prefs.getIntPref("defaultlevel")/100;
            browser.zoompage_imageZoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
            
            /* Initialize not zoomed flag */
            
            browser.zoompage_notZoomed = true;
            
            /* Initialize resize timeout */
            
            browser.zoompage_resizeTimeout = null;
            
            /* Initialize previous uri and previous width/height */
            
            browser.zoompage_previousURI = "";
            browser.zoompage_previousWidth = browser.boxObject.width;
            browser.zoompage_previousHeight = browser.boxObject.height;
            
            /* Initialize last fit-to-width/window zoom values */
            
            browser.zoompage_lastFitToWidth = 1.0;
            browser.zoompage_lastFitToWindow = 1.0;
            
            /* Initialize and remember zoom type for tab */
            
            ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
            browser.zoompage_zoomType = ZoomManager.useFullZoom;
            
            /* Initialize and remember fit-to-window type for tab */
            
            browser.zoompage_ftwType = zoomPage.prefs.getIntPref("imagedefftwtype");
            
            /* Default Zoom - set zoom value for tab opened - page not loaded yet */
            
            if (zoomPage.zoomMode == 0)
            {
                zoomPage.defaultZoom(browser);
                
                zoomPage.setPercent(4);
            }
            
            /* When Firefox starts up, if not forced restart and if start up page is about:blank, */
            /* then for first (only) tab there is no call to onLocationChange() and no pageshow event */
            
            startupPage = zoomPage.otherPrefs.getIntPref("browser.startup.page"); 
            
            if (i == 0 && !zoomPage.forcedRestart && (startupPage == 0 || (startupPage == 1 && gHomeButton.getHomePage() == "about:blank")))
            {
                FullZoom.onLocationChange(browser.currentURI,false,browser);
                
                message.target = browser;
                message.json = new Object();
                message.json.state = true;  /* page loaded */
                
                zoomPage.onPageShow(message);
            }
        }
    },
    
    onUnload: function()
    {
        var i,moreBrowsers,browserWindows,values;
        
        moreBrowsers = false;
        browserWindows = zoomPage.winmed.getEnumerator("navigator:browser");
        while (browserWindows.hasMoreElements()) if (browserWindows.getNext() != window) moreBrowsers = true;
        
        if (!moreBrowsers)  /* last browser */
        {
            /* Enable Firefox's Image Resizing (scaling) preferences */
            
            zoomPage.otherPrefs.setBoolPref("browser.enable_automatic_image_resizing",true);
            zoomPage.otherPrefs.setBoolPref("browser.enable_click_image_resizing",true);
            
            /* Enable Firefox's Site-Specific Zoom preference */
            
            zoomPage.otherPrefs.setBoolPref("browser.zoom.siteSpecific",true);
            
            /* Reinstate Firefox's default zoom global content preference */
            
            zoomPage.setGlobalSiteSpecific(FullZoom.name,1.0);  /* set global zoom value content preference */
            
            /* Reinstate Firefox's standard zoom levels */
            
            zoomPage.levelArray = zoomPage.zoomLevels[0].split("-");
            
            values = "" + zoomPage.levelArray[0]/100;
            for (i = 1; i < zoomPage.levelArray.length; i++) values = values + "," + zoomPage.levelArray[i]/100;
            
            zoomPage.otherPrefs.setIntPref("zoom.minPercent",zoomPage.levelArray[0]);
            zoomPage.otherPrefs.setIntPref("zoom.maxPercent",zoomPage.levelArray[zoomPage.levelArray.length-1]);
            zoomPage.otherPrefs.setCharPref("toolkit.zoomManager.zoomValues",values);
            
            delete ZoomManager.MIN;
            ZoomManager.MIN = zoomPage.levelArray[0]/100;
            delete ZoomManager.MAX;
            ZoomManager.MAX = zoomPage.levelArray[zoomPage.levelArray.length-1]/100;
            ZoomManager.zoomValues.length = 0;
            for (i = 0; i < zoomPage.levelArray.length; i++) ZoomManager.zoomValues[i] = zoomPage.levelArray[i]/100;
        }
    },
    
    /********************************************************************/
    
    /* Redefine existing FullZoom functions */
    
    redefineFullZoomFunctions: function()
    {
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.onContentPrefSet() */
        
        FullZoom.zoompage_old_onContentPrefSet = FullZoom.onContentPrefSet;
        
        FullZoom.onContentPrefSet =
        function new_onContentPrefSet(aGroup, aName, aValue)
        {
            var browser;
            
            browser = gBrowser.selectedBrowser;
            
            zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
            function(browser,sitevalue)
            {
                if (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined")
                {
                    /* In case of changes to global content preference in another browser window */
                    
                    if (aGroup == null) return;
                }
                
                if (!zoomPage.specialPage(browser))  /* not special page */
                {
                    FullZoom._isNextContentPrefChangeInternal = false;  /* FF24+ */
                    
                    FullZoom.zoompage_old_onContentPrefSet(aGroup,aName,aValue);
                }
                
                zoomPage.setPercent(11);
            });
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.onContentPrefRemoved() */
        
        FullZoom.zoompage_old_onContentPrefRemoved = FullZoom.onContentPrefRemoved;
        
        FullZoom.onContentPrefRemoved =
        function new_onContentPrefRemoved(aGroup, aName)
        {
            var browser;
            
            browser = gBrowser.selectedBrowser;
            
            zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
            function(browser,sitevalue)
            {
                if (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined")
                {
                    /* In case of changes to global or site content preferences in another browser window */
                    
                    if (aGroup == null) return;
                    
                    ZoomManager.zoom = browser.zoompage_lastFitToWidth;
                }
                else
                {
                    FullZoom._isNextContentPrefChangeInternal = false;  /* FF24+ */
                    
                    FullZoom.zoompage_old_onContentPrefRemoved(aGroup,aName);
                }
                
                zoomPage.setPercent(12);
            });
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.onLocationChange() */
        
        FullZoom.zoompage_old_onLocationChange = FullZoom.onLocationChange;
        
        FullZoom.onLocationChange =
        function new_onLocationChange(aURI, aIsTabSwitch, aBrowser)
        {
            var browser;
            
            browser = aBrowser || gBrowser.selectedBrowser;  /* browser may be undefined */
            
            zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
            function(browser,sitevalue)
            {
                var i,tab,style,image;
                
                if (aIsTabSwitch)  /* tab select - TabSelect event will be soon before or soon after this call */
                {
                    for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
                        if (gBrowser.tabContainer.childNodes[i].linkedBrowser == browser)
                            tab = gBrowser.tabContainer.childNodes[i];
                    
                    /* Handle pending tab */
                    
                    if (tab.hasAttribute("pending"))
                    {
                        browser.zoompage_pageState = 0;  /* no page */
                        browser.zoompage_previousURI = "";
                        browser.zoompage_lastFitToWidth = zoomPage.lastFitToWidth;
                    }
                    
                    /* Set remembered zoom type for tab */
                    
                    ZoomManager.useFullZoom = browser.zoompage_zoomType;
                    
                    if (zoomPage.imagePage(browser)) ;  /* image page */
                    else  /* not image page */
                    {
                        /* Default Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value in case site-specific zoom removed while tab not selected */
                        
                        if (zoomPage.zoomMode == 2 && typeof sitevalue == "undefined")
                        {
                            zoomPage.defaultZoom(browser);
                        }
                        
                        /* Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set last zoom value in case site-specific zoom removed while tab not selected */
                        
                        if (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined")
                        {
                            if (zoomPage.specialPage(browser))  /* special page - set special zoom */
                            {
                                ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
                            }
                            else  /* normal page - set best guess fit-to-width zoom */
                            {
                                ZoomManager.setZoomForBrowser(browser,browser.zoompage_lastFitToWidth);
                            }
                        }
                        
                        /* Default + Site-Specific Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific set) - apply Firefox site-specific zoom */
                        
                        if ((zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3) && typeof sitevalue != "undefined")
                        {
                            FullZoom.zoompage_old_onLocationChange(aURI,aIsTabSwitch,aBrowser);  /* Firefox site-specific zoom */
                        }
                        
                        if (browser.zoompage_pageState >= 1) zoomPage.setPercent(13);
                    }
                }
                else  /* page load - pageshow event will be soon before or some time after this call */
                {
                    /* Update page state */
                    
                    browser.zoompage_pageState = 1;  /* page loading */
                    
                    if (zoomPage.errorPage(browser))  /* error page */
                    {
                        if (zoomPage.prefs.getBoolPref("specialenable")) ZoomManager.setZoomForBrowser(browser,1.0);
                        else ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("defaultlevel")/100);
                        
                        browser.zoompage_errorShown = true;
                        
                        zoomPage.setPercent(14);
                    }
                    else if (zoomPage.imagePage(browser))  /* image page */
                    {
                        /* Set page and text zoom to 100% */
                        
                        if (zoomPage.ffVersion >= +"29.0")
                        {
                            browser.textZoom = 1;
                            browser.fullZoom = 1;
                        }
                        else
                        {
                            browser.markupDocumentViewer.textZoom = 1;
                            browser.markupDocumentViewer.fullZoom = 1;
                        }
                        
                        /* Set style for image cursor appearance */
                        
                        browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: "default" },{ });
                        
                        /* Set styles for image background appearance */
                        
                        zoomPage.setImageBackground(browser,false);
                        
                        /* Set image page shown state */
                        
                        browser.zoompage_imageShown = false;
                        
                        /* Set error page shown state */
                        
                        browser.zoompage_errorShown = false;
                    }
                    else  /* not image page */
                    {
                        /* Default Zoom - set zoom value for different page loaded */
                        
                        if (zoomPage.zoomMode == 0)
                        {
                            if (zoomPage.specialPage(browser))  /* special page - set special zoom */
                            {
                                ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
                            }
                            else if (browser.zoompage_imageShown || browser.zoompage_specialShown || browser.zoompage_errorShown)  /* leaving image/special/error page - reinstate normal zoom */
                            {
                                ZoomManager.setZoomForBrowser(browser,browser.zoompage_normalZoom);
                            }
                        }
                        
                        /* Default Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for different page loaded */
                        
                        if (zoomPage.zoomMode == 2 && typeof sitevalue == "undefined")
                        {
                            zoomPage.defaultZoom(browser);
                        }
                        
                        /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set last zoom value for different page loaded */
                        
                        if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                        {
                            if (zoomPage.specialPage(browser))  /* special page - set special zoom */
                            {
                                ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
                            }
                            else  /* normal page - set best guess fit-to-width zoom */
                            {
                                if (browser.currentURI.spec != browser.zoompage_previousURI)  /* ignore minor refreshes/updates which don't change uri */
                                    ZoomManager.setZoomForBrowser(browser,browser.zoompage_lastFitToWidth);
                            }
                        }
                        
                        /* Default + Site-Specific Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific set) - apply Firefox site-specific zoom */
                        
                        if ((zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3) && typeof sitevalue != "undefined")
                        {
                            FullZoom.zoompage_old_onLocationChange(aURI,aIsTabSwitch,aBrowser);  /* Firefox site-specific zoom */
                        }
                        
                        /* Set special page shown state */
                        
                        browser.zoompage_specialShown = false;
                        
                        /* Set error page shown state */
                        
                        browser.zoompage_errorShown = false;
                        
                        if (browser.currentURI.spec != "about:blank") zoomPage.setPercent(15);
                    }
                }
            });
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.enlarge() */
        
        FullZoom.zoompage_old_enlarge = FullZoom.enlarge;
        
        FullZoom.enlarge =
        function new_enlarge()
        {
            FullZoom.zoompage_old_enlarge();
            
            zoomPage.setPercent(16);
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.reduce() */
        
        FullZoom.zoompage_old_reduce = FullZoom.reduce;
        
        FullZoom.reduce =
        function new_reduce()
        {
            FullZoom.zoompage_old_reduce();
            
            zoomPage.setPercent(17);
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom.reset() */
        
        FullZoom.reset =
        function new_reset(callback)
        {
            var browser;
            
            ZoomManager.zoom = 1.0;
            
            browser = gBrowser.selectedBrowser;
            
            if (zoomPage.imagePage(browser))  /* image page */
            {
                browser.zoompage_ftwType = zoomPage.prefs.getIntPref("imagedefftwtype");
                
                if (zoomPage.imagezoomMode == 1) zoomPage.fittowindowZoom(browser,false);  /* image - fit-to-window zoom */
                else ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
            }
            else  /* not image page */
            {
                ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                browser.zoompage_zoomType = ZoomManager.useFullZoom;
                
                if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                {
                    zoomPage.fittowidthZoomAndAfter(true,browser,null,"fzr",
                    function(browser,sitevalue)
                    {
                        if (zoomPage.ffVersion >= +"24.0") FullZoom._removePref(browser);
                        else if (zoomPage.ffVersion >= +"22.0") FullZoom._removePref(callback);  /* not remote browser so reference to callback in outer function is okay */
                        else FullZoom._removePref();
                        
                        zoomPage.setPercent(18);
                    });
                }
                else
                {
                    zoomPage.defaultZoom(browser);
                    
                    if (zoomPage.ffVersion >= +"24.0") FullZoom._removePref(browser);
                    else if (zoomPage.ffVersion >= +"22.0") FullZoom._removePref(callback);
                    else FullZoom._removePref();
                    
                    zoomPage.setPercent(19);
                }
            }
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom._applyPrefToZoom() */
        
        if (zoomPage.ffVersion >= +"24.0") FullZoom.zoompage_old_applyPrefToZoom = FullZoom._applyPrefToZoom;
        else FullZoom.zoompage_old_applyPrefToSetting = FullZoom._applyPrefToSetting;
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            FullZoom._applyPrefToZoom =
            function new_applyPrefToZoom(aValue,aBrowser,aCallback)
            {
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.isSyntheticDocument && browser.parentNode)
                    {
                        if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToZoom(aValue,browser,aCallback);
                });
            };
        }
        else if (zoomPage.ffVersion >= +"24.0")
        {
            FullZoom._applyPrefToZoom =
            function new_applyPrefToZoom(aValue,aBrowser,aCallback)
            {
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.contentDocument.mozSyntheticDocument && browser.contentDocument)
                    {
                        if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToZoom(aValue,browser,aCallback);
                });
            };
        }
        else if (zoomPage.ffVersion >= +"22.0")
        {
            FullZoom._applyPrefToSetting =
            function new_applyPrefToSetting(aValue,aBrowser,aCallback)
            {
                aBrowser = aBrowser || (gBrowser && gBrowser.selectedBrowser);
                
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.contentDocument.mozSyntheticDocument)
                    {
                        if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToSetting(aValue,browser,aCallback);
                });
            };
        }
        else if (zoomPage.ffVersion >= +"12.0")
        {
            FullZoom._applyPrefToSetting =
            function new_applyPrefToSetting(aValue,aBrowser)
            {
                aBrowser = aBrowser || (gBrowser && gBrowser.selectedBrowser);
                
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.contentDocument.mozSyntheticDocument)
                    {
                        if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToSetting(aValue,browser);
                });
            };
        }
        else if (zoomPage.ffVersion >= +"9.0")
        {
            FullZoom._applyPrefToSetting =
            function new_applyPrefToSetting(aValue,aBrowser)
            {
                aBrowser = aBrowser || (gBrowser && gBrowser.selectedBrowser);
                
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if (FullZoom.siteSpecific && !gInPrintPreviewMode)
                    {
                        if (browser.contentDocument instanceof Ci.nsIImageDocument) ZoomManager.useFullZoom = 1;
                        else if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToSetting(aValue,browser);
                });
            };
        }
        else
        {
            FullZoom._applyPrefToSetting =
            function new_applyPrefToSetting(aValue,aBrowser)
            {
                aBrowser = aBrowser || (gBrowser && gBrowser.selectedBrowser);
                
                zoomPage.getDomainSiteSpecific(aBrowser,"browser.content.full-type",
                function(browser,sitetype)
                {
                    if ((FullZoom.siteSpecific || FullZoom._inPrivateBrowsing) && !gInPrintPreviewMode)
                    {
                        if (browser.contentDocument instanceof Ci.nsIImageDocument || FullZoom._inPrivateBrowsing) ZoomManager.useFullZoom = 1;
                        else if (typeof sitetype != "undefined") ZoomManager.useFullZoom = sitetype;
                        else ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
                        
                        browser.zoompage_zoomType = ZoomManager.useFullZoom;
                    }
                    
                    FullZoom.zoompage_old_applyPrefToSetting(aValue,browser);
                });
            };
        }
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom._applyZoomToPref() */
        
        if (zoomPage.ffVersion >= +"24.0") FullZoom.zoompage_old_applyZoomToPref = FullZoom._applyZoomToPref;
        else FullZoom.zoompage_old_applySettingToPref = FullZoom._applySettingToPref;
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            FullZoom._applyZoomToPref =
            function new_applyZoomToPref(browser)
            {
                var context;
                
                if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.isSyntheticDocument)
                {
                    context = browser.loadContext;
                    zoomPage.contentPrefs.set(browser.currentURI.spec,"browser.content.full-type",ZoomManager.useFullZoom,context);
                }
                
                FullZoom.zoompage_old_applyZoomToPref(browser);
            };
        }
        else if (zoomPage.ffVersion >= +"24.0")
        {
            FullZoom._applyZoomToPref =
            function new_applyZoomToPref(browser)
            {
                var context;
                
                if (FullZoom.siteSpecific && !gInPrintPreviewMode && !browser.contentDocument.mozSyntheticDocument)
                {
                    context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                              .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
                    zoomPage.contentPrefs.set(browser.currentURI.spec,"browser.content.full-type",ZoomManager.useFullZoom,context);
                }
                
                FullZoom.zoompage_old_applyZoomToPref(browser);
            };
        }
        else if (zoomPage.ffVersion >= +"22.0")
        {
            FullZoom._applySettingToPref =
            function new_applySettingToPref(suppressZoomChange,callback)
            {
                var context;
                
                if (FullZoom.siteSpecific && !gInPrintPreviewMode && !content.document.mozSyntheticDocument)
                {
                    context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                              .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
                    zoomPage.contentPrefs.set(browser.currentURI.spec,"browser.content.full-type",ZoomManager.useFullZoom,context);
                }
                
                FullZoom.zoompage_old_applySettingToPref(suppressZoomChange,callback);
            };
        }
        else if (zoomPage.ffVersion >= +"12.0")
        {
            FullZoom._applySettingToPref =
            function new_applySettingToPref()
            {
                if (FullZoom.siteSpecific && !gInPrintPreviewMode && !content.document.mozSyntheticDocument)
                {
                    zoomPage.contentPrefs.setPref(gBrowser.currentURI,"browser.content.full-type",ZoomManager.useFullZoom);
                }
                
                FullZoom.zoompage_old_applySettingToPref();
            };
        }
        else
        {
            FullZoom._applySettingToPref =
            function new_applySettingToPref()
            {
                if (FullZoom.siteSpecific && !gInPrintPreviewMode && !(content.document instanceof Ci.nsIImageDocument))
                {
                    zoomPage.contentPrefs.setPref(gBrowser.currentURI,"browser.content.full-type",ZoomManager.useFullZoom);
                }
                
                FullZoom.zoompage_old_applySettingToPref();
            };
        }
        
        /*--------------------------------------------------------------*/
        
        /* Redefine FullZoom._removePref() */
        
        FullZoom.zoompage_old_removePref = FullZoom._removePref;
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            FullZoom._removePref =
            function new_removePref(browser)
            {
                var context;
                
                if (!browser.isSyntheticDocument)
                {
                    context = browser.loadContext;
                    zoomPage.contentPrefs.removeByDomainAndName(browser.currentURI.spec,"browser.content.full-type",context);
                }
                
                FullZoom.zoompage_old_removePref(browser);
            };
        }
        else if (zoomPage.ffVersion >= +"24.0")
        {
            FullZoom._removePref =
            function new_removePref(browser)
            {
                var context; 
                
                if (!browser.contentDocument.mozSyntheticDocument)
                {
                    context = browser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                              .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
                    zoomPage.contentPrefs.removeByDomainAndName(browser.currentURI.spec,"browser.content.full-type",context);
                }
                
                FullZoom.zoompage_old_removePref(browser);
            };
        }
        else if (zoomPage.ffVersion >= +"22.0")
        {
            FullZoom._removePref =
            function new_removePref(callback)
            {
                var context;
                
                if (!content.document.mozSyntheticDocument)
                {
                    context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                              .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
                    zoomPage.contentPrefs.removeByDomainAndName(browser.currentURI.spec,"browser.content.full-type",context);
                }
                
                FullZoom.zoompage_old_removePref(callback);
            };
        }
        else if (zoomPage.ffVersion >= +"12.0")
        {
            FullZoom._removePref =
            function new_removePref()
            {
                if (!content.document.mozSyntheticDocument)
                {
                    zoomPage.contentPrefs.removePref(gBrowser.currentURI,"browser.content.full-type");
                }
                
                FullZoom.zoompage_old_removePref();
            };
        }
        else
        {
            FullZoom._removePref =
            function new_removePref()
            {
                if (!(content.document instanceof Ci.nsIImageDocument))
                {
                    zoomPage.contentPrefs.removePref(gBrowser.currentURI,"browser.content.full-type");
                }
                
                FullZoom.zoompage_old_removePref();
            };
        }
        
        /*--------------------------------------------------------------*/
    },
    
    /********************************************************************/
    
    /* Redefine existing ZoomManager functions */
    
    redefineZoomManagerFunctions: function()
    {
        /*--------------------------------------------------------------*/
        
        /* Redefine ZoomManager.getZoomForBrowser() */
        
        ZoomManager.zoompage_old_getZoomForBrowser = ZoomManager.getZoomForBrowser;
        
        ZoomManager.getZoomForBrowser =
        function new_getZoomForBrowser(aBrowser)
        {
            if (aBrowser == gBrowser) aBrowser = gBrowser.selectedBrowser;  /* because 'get zoom()' in Zoom Manager sets aBrowser to getBrowser() */
            
            if (zoomPage.imagePage(aBrowser))  /* image page */
            {
                return aBrowser.zoompage_imageZoom;
            }
            else  /* not image page */
            {
                return ZoomManager.zoompage_old_getZoomForBrowser(aBrowser);
            }
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine ZoomManager.setZoomForBrowser() */
        
        ZoomManager.setZoomForBrowser =
        function new_setZoomForBrowser(aBrowser, aVal)
        {
            var image,style;
            
            if (aBrowser == gBrowser) aBrowser = gBrowser.selectedBrowser;  /* because 'set zoom()' in Zoom Manager sets aBrowser to getBrowser() */
            
            if (zoomPage.imagePage(aBrowser))  /* image page */
            {
                if (aVal < 1/100 || aVal > 10000/100)
                    throw Components.results.NS_ERROR_INVALID_ARG;
                
                if (zoomPage.ffVersion >= +"29.0" || zoomPage.ffPaleMoon)
                {
                    aBrowser.textZoom = 1;
                    aBrowser.fullZoom = 1;
                }
                else
                {
                    aBrowser.markupDocumentViewer.textZoom = 1;
                    aBrowser.markupDocumentViewer.fullZoom = 1;
                }
                
                aBrowser.messageManager.sendAsyncMessage("zoompage-setimagezoom",{ width: aBrowser.zoompage_imageWidth*aVal, height: aBrowser.zoompage_imageHeight*aVal },{ });
                
                aBrowser.zoompage_imageZoom = aVal;
                
                aBrowser.zoompage_notZoomed = false;
                
                zoomPage.setImageBackground(aBrowser,true);
                
                zoomPage.setImageCursor(aBrowser);
            }
            else  /* not image page */
            {
                if (aVal < ZoomManager.MIN || aVal > ZoomManager.MAX)
                    throw Components.results.NS_ERROR_INVALID_ARG;
                
                if (zoomPage.ffVersion >= +"29.0" || zoomPage.ffPaleMoon /* PM28+ */)
                {
                    if (ZoomManager.useFullZoom || aBrowser.isSyntheticDocument /* FF29+ */)
                    {
                        if (Math.round(aVal*100) >= zoomPage.prefs.getIntPref("textminlevel")) aBrowser.textZoom = 1;
                        else aBrowser.textZoom = (zoomPage.prefs.getIntPref("textminlevel")/100)/aVal;
                        aBrowser.fullZoom = aVal;
                    }
                    else
                    {
                        aBrowser.textZoom = aVal;
                        aBrowser.fullZoom = 1;
                    }
                }
                else
                {
                    if (ZoomManager.useFullZoom || aBrowser.contentDocument && aBrowser.contentDocument.mozSyntheticDocument /* FF8-FF28 */)
                    {
                        if (Math.round(aVal*100) >= zoomPage.prefs.getIntPref("textminlevel")) aBrowser.markupDocumentViewer.textZoom = 1;
                        else aBrowser.markupDocumentViewer.textZoom = (zoomPage.prefs.getIntPref("textminlevel")/100)/aVal;
                        aBrowser.markupDocumentViewer.fullZoom = aVal;
                    }
                    else
                    {
                        aBrowser.markupDocumentViewer.textZoom = aVal;
                        aBrowser.markupDocumentViewer.fullZoom = 1;
                    }
                }
                
                if (!zoomPage.errorPage(aBrowser) && !zoomPage.specialPage(aBrowser)) aBrowser.zoompage_normalZoom = aVal;
                
                aBrowser.zoompage_notZoomed = false;
            }
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine ZoomManager.enlarge() */
        
        ZoomManager.enlarge =
        function new_enlarge(shiftKey)
        {
            var i,step;
            
            /* shiftKey is not normally present - indicates Ctrl+Shift+"="/"+" shortcut */
            
            step = zoomPage.prefs.getIntPref("zoomstep");
            
            if ((zoomPage.prefs.getBoolPref("usezoomstep") && shiftKey == null) ||
                (!zoomPage.prefs.getBoolPref("usezoomstep") && shiftKey == true))
            {
                if (this.zoom <= (10000-step)/100) this.zoom += step/100;
            }
            else
            {
                for (i = 0; i < this.zoomValues.length && Math.round(this.zoom*100)/100 >= this.zoomValues[i]; i++) ;
                
                if (i < this.zoomValues.length) this.zoom = this.zoomValues[i];
            }
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine ZoomManager.reduce() */
        
        ZoomManager.reduce =
        function new_reduce(shiftKey)
        {
            var i,step;
            
            /* shiftKey is not normally present - indicates Ctrl+Shift+"-"/"_" shortcut */
            
            step = zoomPage.prefs.getIntPref("zoomstep");
            
            if ((zoomPage.prefs.getBoolPref("usezoomstep") && shiftKey == null) ||
                (!zoomPage.prefs.getBoolPref("usezoomstep") && shiftKey == true))
            {
                if (this.zoom >= (1+step)/100) this.zoom -= step/100;
            }
            else
            {
                for (i = this.zoomValues.length-1; i >= 0 && Math.round(this.zoom*100)/100 <= this.zoomValues[i]; i--) ;
                
                if (i >= 0) this.zoom = this.zoomValues[i];
            }
        };
        
        /*--------------------------------------------------------------*/
        
        /* Redefine ZoomManager.toggleZoom() */
        
        ZoomManager.zoompage_old_toggleZoom = ZoomManager.toggleZoom;
        
        ZoomManager.toggleZoom =
        function new_toggleZoom()
        {
            var browser;
            
            ZoomManager.zoompage_old_toggleZoom();
            
            browser = gBrowser.selectedBrowser;
            
            browser.zoompage_zoomType = ZoomManager.useFullZoom;
            
            zoomPage.getDomainSiteSpecific(gBrowser.selectedBrowser,FullZoom.name,
            function(browser,sitevalue)
            {
                var context,doftw;
                
                /* Remember site-specific zoom type and zoom value */
                
                if (zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3)
                {
                    if (zoomPage.ffVersion >= +"29.0")
                    {
                        context = browser.loadContext;
                        zoomPage.contentPrefs.set(browser.currentURI.spec,"browser.content.full-type",ZoomManager.useFullZoom,context);
                        zoomPage.contentPrefs.set(browser.currentURI.spec,FullZoom.name,ZoomManager.getZoomForBrowser(browser),context);
                    }
                    else if (zoomPage.ffVersion >= +"22.0")
                    {
                        context = browser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                                  .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
                        zoomPage.contentPrefs.set(browser.currentURI.spec,"browser.content.full-type",ZoomManager.useFullZoom,context);
                        zoomPage.contentPrefs.set(browser.currentURI.spec,FullZoom.name,ZoomManager.getZoomForBrowser(browser),context);
                    }
                    else
                    {
                        zoomPage.contentPrefs.setPref(gBrowser.currentURI,"browser.content.full-type",ZoomManager.useFullZoom);
                        zoomPage.contentPrefs.setPref(gBrowser.currentURI,FullZoom.name,ZoomManager.getZoomForBrowser(browser));
                    }
                }
                
                /* Update fit-to-width zoom value */
                
                if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                {
                    doftw = true;
                }
                
                zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"zmtz",
                function(browser,sitevalue)
                {
                    zoomPage.setPercent(21);
                });
            });
        };
        
        /*--------------------------------------------------------------*/
    },
    
    /********************************************************************/
    
    /* Display functions */
    
    activeButtonStyle: function()
    {
        if (window.getComputedStyle(document.getElementById("zoompage-style-c"),null).getPropertyValue("display") != "none") return "c";
        if (window.getComputedStyle(document.getElementById("zoompage-style-n"),null).getPropertyValue("display") != "none") return "n";
        if (window.getComputedStyle(document.getElementById("zoompage-style-s"),null).getPropertyValue("display") != "none") return "s";
        if (window.getComputedStyle(document.getElementById("zoompage-style-t"),null).getPropertyValue("display") != "none") return "t";
        if (window.getComputedStyle(document.getElementById("zoompage-style-m"),null).getPropertyValue("display") != "none") return "m";
        
        return "n";
    },
    
    setPercent: function(trace)
    {
        if (document.getElementById("main-window").hasAttribute("customizing")) return;
        
        /* Allow time for zooming to complete - complex pages are slower to zoom */
        /* 10ms is usually enough without e10s - 100ms is usually enough with e10s */
        
        window.setTimeout(function() { zoomPage.setPercentDelayed(trace); },100);
    },
    
    setPercentDelayed: function(trace)
    {
        var browser,percent,string;
        
        if (document.getElementById("zoompage-item") != null)  /* button accessible */
        {
            browser = gBrowser.selectedBrowser;
            
            if (zoomPage.imagePage(browser))  /* image page */
            {
                /* Initialize */
                
                percent = document.getElementById("zoompage-percent-label-" + zoomPage.activeButtonStyle());
                
                /* Adjust display field width */
                
                zoomPage.calculateDisplayWidth();
                
                /* Determine active indicators */
                
                percent.style.removeProperty("font-style");
                percent.style.removeProperty("color");
                percent.style.removeProperty("outline-width");
                percent.style.removeProperty("outline-style");
                percent.style.removeProperty("outline-offset");
                
                string = "" + Math.round(ZoomManager.zoom*100);
                
                /* Explicit / Default / Fit-To-Width indicators */
                
                if (browser.zoompage_notZoomed)
                {
                    if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                }
                else if (zoomPage.imagezoomMode == 0)
                {
                    if (Math.round(ZoomManager.zoom*100) == zoomPage.prefs.getIntPref("imgdefaultlevel"))
                    {
                        if (zoomPage.prefs.getBoolPref("defprefix")) string = document.getElementById("zoompage-defflag").value + string;
                        if (zoomPage.prefs.getBoolPref("defborder"))
                        {
                            percent.style.setProperty("outline-width","1px","");
                            percent.style.setProperty("outline-style","dotted","");
                            percent.style.setProperty("outline-offset","-1px","");
                        }
                    }
                    else if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                }
                else if (zoomPage.imagezoomMode == 1)
                {
                    if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWindow*100))
                    {
                        if (zoomPage.prefs.getBoolPref("ftwprefix")) string = document.getElementById("zoompage-ftwflag").value + string;
                        if (zoomPage.prefs.getBoolPref("ftwborder"))
                        {
                            percent.style.setProperty("outline-width","1px","");
                            percent.style.setProperty("outline-style","dashed","");
                            percent.style.setProperty("outline-offset","-1px","");
                        }
                    }
                    else if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                }
                
                /* Percent indicator */
                
                if (zoomPage.prefs.getBoolPref("pcsuffix")) string = string + "%";
                
                /* Trace source of setPercent() */
                
                if (zoomPage.traceEnable) string = trace + " - " + string;
                
                /* Call history for setPercent() */
                
                if (zoomPage.historyEnable)
                {
                    zoomPage.historyString += " | " + (zoomPage.historyCount++) + " - " + string;
                    zoomPage.historyString = zoomPage.historyString.substr(-200);
                    zoomPage.historyString = zoomPage.historyString.substr(zoomPage.historyString.indexOf("|"));
                    zoomPage.debugMessage("Zoom History",zoomPage.historyString);
                }
                
                /* Update zoom percentage and indicators */
                
                percent.setAttribute("value",string);
            }
            else  /* not image page */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    var percent,string;
                    
                    /* Initialize */
                    
                    percent = document.getElementById("zoompage-percent-label-" + zoomPage.activeButtonStyle());
                    
                    /* Adjust display field width */
                    
                    zoomPage.calculateDisplayWidth();
                    
                    /* Determine active indicators */
                    
                    percent.style.removeProperty("font-style");
                    percent.style.removeProperty("color");
                    percent.style.removeProperty("outline-width");
                    percent.style.removeProperty("outline-style");
                    percent.style.removeProperty("outline-offset");
                    
                    string = "" + Math.round(ZoomManager.zoom*100);
                    
                    /* Full-Page / Text-Only indicators */
                    
                    if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full"))
                    {
                        if (zoomPage.prefs.getBoolPref("fpprefix")) string = document.getElementById("zoompage-fpflag").value + string;
                    }
                    else
                    {
                        if (zoomPage.prefs.getBoolPref("toprefix")) string = document.getElementById("zoompage-toflag").value + string;
                        if (zoomPage.prefs.getBoolPref("toitalic")) percent.style.setProperty("font-style","italic","");
                        if (zoomPage.prefs.getBoolPref("tored")) percent.style.setProperty("color","#800000","");
                    }
                    
                    /* Explicit / Default / Fit-To-Width / Site-Specific indicators */
                    
                    if (browser.zoompage_notZoomed)
                    {
                        if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                    }
                    else if ((zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3) && typeof sitevalue != "undefined" && Math.round(ZoomManager.zoom*100) == Math.round(sitevalue*100))  /* not if ignoring Site-Specific zoom level */
                    {
                        if (zoomPage.prefs.getBoolPref("ssprefix")) string = document.getElementById("zoompage-ssflag").value + string;
                        if (zoomPage.prefs.getBoolPref("ssborder"))
                        {
                            percent.style.setProperty("outline-width","1px","");
                            percent.style.setProperty("outline-style","solid","");
                            percent.style.setProperty("outline-offset","-1px","");
                        }
                    }
                    else if (zoomPage.zoomMode == 0 || zoomPage.zoomMode == 2)
                    {
                        if (Math.round(ZoomManager.zoom*100) == zoomPage.defaultLevel(browser))
                        {
                            if (zoomPage.prefs.getBoolPref("defprefix")) string = document.getElementById("zoompage-defflag").value + string;
                            if (zoomPage.prefs.getBoolPref("defborder"))
                            {
                                percent.style.setProperty("outline-width","1px","");
                                percent.style.setProperty("outline-style","dotted","");
                                percent.style.setProperty("outline-offset","-1px","");
                            }
                        }
                        else if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                    }
                    else if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                    {
                        if (zoomPage.specialPage(browser))
                        {
                            if (Math.round(ZoomManager.zoom*100) == zoomPage.defaultLevel(browser))
                            {
                                if (zoomPage.prefs.getBoolPref("defprefix")) string = document.getElementById("zoompage-defflag").value + string;
                                if (zoomPage.prefs.getBoolPref("defborder"))
                                {
                                    percent.style.setProperty("outline-width","1px","");
                                    percent.style.setProperty("outline-style","dotted","");
                                    percent.style.setProperty("outline-offset","-1px","");
                                }
                            }
                            else if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                        }
                        else
                        {
                            if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWidth*100))
                            {
                                if (zoomPage.prefs.getBoolPref("ftwprefix")) string = document.getElementById("zoompage-ftwflag").value + string;
                                if (zoomPage.prefs.getBoolPref("ftwborder"))
                                {
                                    percent.style.setProperty("outline-width","1px","");
                                    percent.style.setProperty("outline-style","dashed","");
                                    percent.style.setProperty("outline-offset","-1px","");
                                }
                            }
                            else if (zoomPage.prefs.getBoolPref("expprefix")) string = document.getElementById("zoompage-expflag").value + string;
                        }
                    }
                    
                    /* Percent indicator */
                    
                    if (zoomPage.prefs.getBoolPref("pcsuffix")) string = string + "%";
                    
                    /* Trace source of setPercent() */
                    
                    if (zoomPage.traceEnable) string = trace + " - " + string;
                    
                    /* Call history for setPercent() */
                    
                    if (zoomPage.historyEnable)
                    {
                        zoomPage.historyString += " | " + (zoomPage.historyCount++) + " - " + string;
                        zoomPage.historyString = zoomPage.historyString.substr(-200);
                        zoomPage.historyString = zoomPage.historyString.substr(zoomPage.historyString.indexOf("|"));
                        zoomPage.debugMessage("Zoom History",zoomPage.historyString);
                    }
                    
                    /* Update zoom percentage and indicators */
                    
                    percent.setAttribute("value",string);
                });
            }
        }
    },
    
    calculateDisplayWidth: function()
    {
        var percent,numW,expW,defW,ftwW,ssW,fpW,toW,pcW,maxW1,maxW2,maxW3;
        
        percent = document.getElementById("zoompage-percent-label-" + zoomPage.activeButtonStyle());
        
        /* Calculate field width */
        
        percent.setAttribute("value","");
        percent.style.removeProperty("width");
        percent.style.setProperty("font-style","italic","");
        percent.style.setProperty("font-weight","bold","");
        
        percent.setAttribute("value","888");
        numW = percent.boxObject.width;
        
        percent.setAttribute("value",document.getElementById("zoompage-expflag").value + "888");
        if (zoomPage.prefs.getBoolPref("expprefix")) expW = percent.boxObject.width; else expW = numW;
        percent.setAttribute("value",document.getElementById("zoompage-defflag").value + "888");
        if (zoomPage.prefs.getBoolPref("defprefix")) defW = percent.boxObject.width; else defW = numW;
        percent.setAttribute("value",document.getElementById("zoompage-ftwflag").value + "888");
        if (zoomPage.prefs.getBoolPref("ftwprefix")) ftwW = percent.boxObject.width; else ftwW = numW;
        percent.setAttribute("value",document.getElementById("zoompage-ssflag").value + "888");
        if (zoomPage.prefs.getBoolPref("ssprefix")) ssW = percent.boxObject.width; else ssW = numW;
        maxW1 = Math.max(expW,defW,ftwW,ssW)-numW;
        
        percent.setAttribute("value",document.getElementById("zoompage-fpflag").value + "888");
        if (zoomPage.prefs.getBoolPref("fpprefix")) fpW = percent.boxObject.width; else fpW = numW;
        percent.setAttribute("value",document.getElementById("zoompage-toflag").value + "888");
        if (zoomPage.prefs.getBoolPref("toprefix")) toW = percent.boxObject.width; else toW = numW;
        maxW2 = Math.max(fpW,toW)-numW;
        
        percent.setAttribute("value","888" + "%");
        if (zoomPage.prefs.getBoolPref("pcsuffix")) pcW = percent.boxObject.width; else pcW = numW;
        maxW3 = pcW-numW;
        
        percent.style.setProperty("width",(maxW1+maxW2+numW+maxW3+4)+"px","important");
        
        percent.style.removeProperty("font-style");
        percent.style.removeProperty("font-weight");
    },
    
    setImageBackground: function(browser,imageshown)
    {
        browser.messageManager.sendAsyncMessage("zoompage-setimagebackground",{ imageshown: imageshown },{ });
    },
    
    setImageCursor: function(browser)
    {
        zoomPage.setImageCursorDelayed(browser);  /* immediately */

        window.setTimeout(function(browser) { zoomPage.setImageCursorDelayed(browser); },100,browser);  /* allow time for zooming to complete */
    },
    
    setImageCursorDelayed: function(browser)
    {
        var imageWidth,cursor;
        
        if (!zoomPage.prefs.getBoolPref("imgclickftwdef"))
        {
            /* Set default style for image cursor appearance */
            
            browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: "default" },{ });
        }
        else
        {
            /* Determine style for image cursor appearance */
            
            imageWidth = browser.zoompage_imageWidth*ZoomManager.getZoomForBrowser(browser);
            
            if (zoomPage.imagezoomMode == 0)
            {
                if (Math.round(ZoomManager.getZoomForBrowser(browser)*100) == zoomPage.prefs.getIntPref("imgdefaultlevel"))  /* default size */
                {
                    browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: "expand-shrink" },{ });
                }
                else  /* any other size */
                {
                    if (imageWidth > browser.zoompage_imageWidth*zoomPage.prefs.getIntPref("imgdefaultlevel")/100) cursor = "zoomout";
                    else cursor = "zoomin";
                    
                    browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: cursor },{ });
                }
            }
            else
            {
                if (Math.round(ZoomManager.getZoomForBrowser(browser)*100) == Math.round(browser.zoompage_lastFitToWindow*100))  /* fit-to-window size */
                {
                    if (imageWidth > browser.zoompage_imageWidth*zoomPage.prefs.getIntPref("imgdefaultlevel")/100) cursor = "zoomout";
                    else cursor = "zoomin";
                    
                    browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: cursor },{ });
                }
                else  /* any other size */
                {
                    browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: "expand-shrink" },{ });
                }
            }
            
            /* Set style for image cursor appearance */
            
            browser.messageManager.sendAsyncMessage("zoompage-setimagecursor",{ cursor: cursor },{ });
        }
    },
    
    showStatusPanel: function(type,sitevalue)
    {
        var i,browser,panel,label,mode;
        
        browser = gBrowser.selectedBrowser;
        
        panel = document.getElementById("zoompage-modestatuspanel");
        
        if (zoomPage.modestatusInterval != null)
        {
            window.clearInterval(zoomPage.modestatusInterval);
            zoomPage.modestatusInterval = null;
            panel.hidePopup();
        }
        
        if (panel.childNodes.length > 0)
        {
            label = panel.childNodes[0];
            panel.removeChild(label);
        }
        
        label = document.createElement("label");
        panel.appendChild(label);
        panel.style.setProperty("min-width","40px","");
        panel.style.setProperty("font-weight","bold","");
        panel.style.setProperty("text-align","center","");
        
        if (zoomPage.imagePage(browser)) mode = zoomPage.imagezoomMode;
        else mode = zoomPage.zoomMode;
        
        if (type == 0)  /* D/X or F/X or D/S or F/S */
        {
            if (mode == 0) label.setAttribute("value",document.getElementById("zoompage-defflag").value + "/" + document.getElementById("zoompage-expflag").value);
            if (mode == 1) label.setAttribute("value",document.getElementById("zoompage-ftwflag").value + "/" + document.getElementById("zoompage-expflag").value);
            if (mode == 2) label.setAttribute("value",document.getElementById("zoompage-defflag").value + "/" + document.getElementById("zoompage-ssflag").value);
            if (mode == 3) label.setAttribute("value",document.getElementById("zoompage-ftwflag").value + "/" + document.getElementById("zoompage-ssflag").value);
        }
        else if (type == 1)  /* D or F - image page */
        {
            if (mode == 0 || mode == 2)
            {
                if (Math.round(ZoomManager.zoom*100) == zoomPage.prefs.getIntPref("imgdefaultlevel"))
                    label.setAttribute("value",document.getElementById("zoompage-defflag").value);
                else label.setAttribute("value",document.getElementById("zoompage-ftwflag").value);
            }
            else
            {
                if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWindow*100))
                    label.setAttribute("value",document.getElementById("zoompage-ftwflag").value);
                else label.setAttribute("value",document.getElementById("zoompage-defflag").value);
            }
        }
        else if (type == 2)  /* D or F - not image page */
        {
            if (mode == 0 || mode == 2)
            {
                if (Math.round(ZoomManager.zoom*100) == zoomPage.defaultLevel(browser))
                    label.setAttribute("value",document.getElementById("zoompage-defflag").value);
                else label.setAttribute("value",document.getElementById("zoompage-ftwflag").value);
            }
            else
            {
                if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWidth*100))
                    label.setAttribute("value",document.getElementById("zoompage-ftwflag").value);
                else label.setAttribute("value",document.getElementById("zoompage-defflag").value);
            }
        }
        else if (type == 3)  /* S or $ */
        {
            if (mode == 2 || mode == 3)
            {
                if (typeof sitevalue != "undefined")
                {
                    if (Math.round(ZoomManager.zoom*100) == Math.round(sitevalue*100)) label.setAttribute("value",document.getElementById("zoompage-ssflag").value);
                    else label.setAttribute("value","$");
                }
                else  label.setAttribute("value","$");
            }
        }
        else if (type == 4)  /* P or T */
        {
            if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full")) label.setAttribute("value",document.getElementById("zoompage-fpflag").value);
            else label.setAttribute("value",document.getElementById("zoompage-toflag").value);
        }
        else if (type == 5)  /* 1 or 2 */
        {
            if (browser.zoompage_ftwType == 0) label.setAttribute("value",document.getElementById("zoompage-1dflag").value);
            else label.setAttribute("value",document.getElementById("zoompage-2dflag").value);
        }
        
        panel.openPopup(browser,"overlap",browser.clientWidth/2-20,5,false,false,null);
        
        zoomPage.modestatusCount = 100;
        
        zoomPage.modestatusInterval = window.setInterval(function(panel)
        {
            if (zoomPage.modestatusCount-- > 50) panel.style.setProperty("opacity","1.0","");
            else if (zoomPage.modestatusCount-- > 0) panel.style.setProperty("opacity",zoomPage.modestatusCount*0.02,"");
            else
            {
                window.clearInterval(zoomPage.modestatusInterval);
                zoomPage.modestatusInterval = null;
                panel.hidePopup();
            }
        },20,panel);  /* 100 * 20ms = 2 second fade */
    },
    
    /********************************************************************/
    
    /* Zoom functions */
    
    errorPage: function(browser)
    {
        if (typeof browser.currentURI == "undefined")  /* should not happen */
        {
            return true;
        }
        else if (((zoomPage.ffVersion >= +"25.0" && !zoomPage.ffPaleMoon) && browser.documentURI && browser.documentURI.spec.substr(0,14) == "about:neterror") ||
                 ((zoomPage.ffVersion < +"25.0" || zoomPage.ffPaleMoon) && browser.contentDocument && browser.contentDocument.documentURI.substr(0,14) == "about:neterror"))  /* network error */
        {
            return true;
        }
        else if (((zoomPage.ffVersion >= +"25.0" && !zoomPage.ffPaleMoon) && browser.documentURI && browser.documentURI.spec.substr(0,15) == "about:certerror") ||
                 ((zoomPage.ffVersion < +"25.0" || zoomPage.ffPaleMoon) && browser.contentDocument && browser.contentDocument.documentURI.substr(0,15) == "about:certerror"))  /* certificate error */
        {
            return true;
        }
        else if (((zoomPage.ffVersion >= +"25.0" && !zoomPage.ffPaleMoon) && browser.documentURI && browser.documentURI.spec.substr(0,13) == "about:blocked") ||
                 ((zoomPage.ffVersion < +"25.0" || zoomPage.ffPaleMoon) && browser.contentDocument && browser.contentDocument.documentURI.substr(0,13) == "about:blocked"))  /* blocked error */
        {
            return true;
        }
        else return false;
    },
    
    imagePage: function(browser)
    {
        if ((zoomPage.ffVersion >= +"30.0" && browser.documentContentType && browser.documentContentType.substr(0,5) == "image" && browser.documentContentType != "image/svg+xml") ||
            (zoomPage.ffVersion < +"30.0" && browser.contentDocument && browser.contentDocument instanceof Components.interfaces.nsIImageDocument))  /* image document */
        {
            return true;
        }
        else return false;
    },
    
    specialPage: function(browser)
    {
        if (document.getElementById("print-preview-toolbar") != null)  /* print preview mode */
        {
            return 2;
        }
        else if (document.getElementById("printedit-toolbar") != null &&
                 document.getElementById("printedit-toolbar").getAttribute("collapsed") == "false")  /* print edit mode */
        {
            return 2;
        }
        else if (browser.currentURI.spec.substr(0,6) == "about:" &&
                 (browser.currentURI.spec != "about:blank" || browser.zoompage_pageState == 2))  /* Firefox about page - ignore about:blank if page not loaded */
        {
            return 1;
        }
        else if ((zoomPage.ffVersion >= +"29.0" && browser.isSyntheticDocument) ||
                 (zoomPage.ffVersion < +"29.0" && browser.contentDocument && browser.contentDocument.mozSyntheticDocument))  /* media document (video/audio/image) */
        {
            return 1;
        }
        else if ((zoomPage.ffVersion >= +"30.0" && browser.documentContentType && browser.documentContentType == "application/vnd.mozilla.xul+xml") ||
                 (zoomPage.ffVersion < +"30.0" && browser.contentDocument && browser.contentDocument.documentElement instanceof XULElement))  /* Mozilla XUL document */
        {
            return 1;
        }
        else  /* normal page */
        {
            return 0;
        }
    },
    
    defaultLevel: function(browser)
    {
        switch (zoomPage.specialPage(browser))
        {
            case 0:  /* normal page */
                return zoomPage.prefs.getIntPref("defaultlevel");
            
            case 1:  /* audio, video, about: or xul page */
                if (zoomPage.prefs.getBoolPref("specialenable")) return 100;
                else return zoomPage.prefs.getIntPref("defaultlevel");
            
            case 2:  /* print preview or print edit page */
                return 100;
        }
    },
    
    defaultZoom: function(browser)
    {
        if (zoomPage.specialPage(browser))  /* special page - set special zoom */
        {
            ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
        }
        else  /* normal page - set default zoom */
        {
            ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("defaultlevel")/100);
        }
    },
    
    fittowidthZoomAndAfter: function(doftw,browser,sitevalue,reference,callback)
    {
        if (doftw)
        {
            if (zoomPage.specialPage(browser))  /* special page - set special zoom */
            {
                if (reference == "ops" || reference == "ohc" || reference == "ots" || reference == "obr")  /* automatic operations */
                {
                    ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
                    
                    /* Remember fit-to-width zoom level */
                    
                    browser.zoompage_lastFitToWidth = ZoomManager.getZoomForBrowser(browser);
                }
                
                callback(browser,sitevalue);
            }
            else  /* normal page - set fit-to-width zoom */
            {
                zoomPage.fittowidthZoom(browser,sitevalue,reference,callback);
            }
        }
        else callback(browser,sitevalue);
    },
    
    fittowidthZoom: function(browser,sitevalue,reference,callback)
    {
        var min,max,ful,ratio,browserMM;
        
        /* Calculate max zoom level */
        
        min = zoomPage.prefs.getIntPref("fittowidthmin")/100;
        max = zoomPage.prefs.getIntPref("fittowidthmax")/100;
        ful = zoomPage.prefs.getIntPref("fittowidthful")/100;
        
        if (browser.boxObject.width > window.screen.width/2)
        {
            ratio = (browser.boxObject.width-window.screen.width/2)/(window.screen.width/2);
            max += (ful-max)*ratio;
        }
        
        /* Remember callback function */
        
        if (zoomPage.ftwCallbackRef.indexOf(reference) == -1)
        {
            zoomPage.ftwCallbackRef.push(reference);
            zoomPage.ftwCallbackFunc.push(callback);
        }
        
        /* Send message to browser framescript */
        
        browserMM = browser.messageManager;
        
        browserMM.sendAsyncMessage("zoompage-fittowidthzoom",{ sitevalue: sitevalue, reference: reference, min: min, max: max });
    },
    
    fittowidthZoom_Reply: function(message)
    {
        var browser,index;
        
        browser = message.target;
        
        /* Remember fit-to-width zoom level */
        
        browser.zoompage_lastFitToWidth = ZoomManager.getZoomForBrowser(browser);
        
        zoomPage.lastFitToWidth = ZoomManager.getZoomForBrowser(browser);
        
        /* Call callback function defined in call to fittowidthZoomAndAfter() */
        
        index = zoomPage.ftwCallbackRef.indexOf(message.json.reference);
        
        if (index >= 0) zoomPage.ftwCallbackFunc[index](message.target,message.json.sitevalue);
    },
    
    fittowindowZoom: function(browser,otherType)
    {
        var min,max,imageMargin,ftwType,zoomX,zoomY,zoom;
        
        min = zoomPage.prefs.getIntPref("imgfittowindowmin")/100;
        max = zoomPage.prefs.getIntPref("imgfittowindowmax")/100;
        
        if ((zoomPage.prefs.getIntPref("imageposition") & 0x01) == 1) imageMargin = 10; else imageMargin = 0;
        
        ftwType = otherType ? browser.zoompage_ftwType ^ 0x01 : browser.zoompage_ftwType;
        
        if (ftwType == 0)  /* 1D */
        {
            zoomX = (browser.clientWidth-17-imageMargin*2)/browser.zoompage_imageWidth;
            zoomY = (browser.clientHeight-17-imageMargin*2)/browser.zoompage_imageHeight;
            zoom = Math.max(zoomX,zoomY);
        }
        else  /* 2D */
        {
            zoomX = (browser.clientWidth-imageMargin*2)/browser.zoompage_imageWidth;
            zoomY = (browser.clientHeight-imageMargin*2)/browser.zoompage_imageHeight;
            zoom = Math.min(zoomX,zoomY);
        }
        
        if (zoom < min) zoom = min;
        else if (zoom > max) zoom = max;
        
        ZoomManager.setZoomForBrowser(browser,zoom);
        
        browser.zoompage_lastFitToWindow = zoom;
    },
    
    getDomainSiteSpecific: function(browser,name,callback)
    {
        var context,pref,value;
        
        /* get site-specific zoom value content preference */
        
        if (typeof browser.currentURI == "undefined") return;
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            context = browser.loadContext;
            pref = zoomPage.contentPrefs.getCachedByDomainAndName(browser.currentURI.spec,name,context);
            if (pref) callback(browser,pref.value);
            else
            {
                value = undefined;
                zoomPage.contentPrefs.getByDomainAndName(browser.currentURI.spec,name,context,
                    { handleResult: function (resultPref) { value = resultPref.value; }, handleCompletion: function() { callback(browser,value); } });
            }
        }
        else if (zoomPage.ffVersion >= +"22.0")
        {
            context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
            pref = zoomPage.contentPrefs.getCachedByDomainAndName(browser.currentURI.spec,name,context);
            if (pref) callback(browser,pref.value);
            else
            {
                value = undefined;
                zoomPage.contentPrefs.getByDomainAndName(browser.currentURI.spec,name,context,
                    { handleResult: function (resultPref) { value = resultPref.value; }, handleCompletion: function() { callback(browser,value); } });
            }
        }
        else if (zoomPage.ffVersion >= +"19.0")
        {
            context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
            callback(browser,zoomPage.contentPrefs.getPref(browser.currentURI,name,context));
        }
        else callback(browser,zoomPage.contentPrefs.getPref(browser.currentURI,name));
    },
    
    setGlobalSiteSpecific: function(name,value)
    {
        var context;
        
        /* set global zoom value content preference */
        
        if (zoomPage.ffVersion >= +"29.0")
        {
            context = gBrowser.selectedBrowser.loadContext;
            zoomPage.contentPrefs.setGlobal(name,value,context);
        }
        else if (zoomPage.ffVersion >= +"22.0")
        {
            context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
            zoomPage.contentPrefs.setGlobal(name,value,context);
        }
        else if (zoomPage.ffVersion >= +"19.0")
        {
            context = gBrowser.contentWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIWebNavigation).QueryInterface(Components.interfaces.nsILoadContext);
            zoomPage.contentPrefs.setPref(null,name,value,context);
        }
        else zoomPage.contentPrefs.setPref(null,name,value);
    },
    
    /********************************************************************/
    
    /* Event handlers for user interface events */
    
    onTooltipPopupShowing: function(event)
    {
        var browser;
        
        browser = gBrowser.selectedBrowser;
        
        if (zoomPage.prefs.getBoolPref("swapmouseclicks"))
        {
            document.getElementById("zoompage-zoomdisplay-tip-1a").setAttribute("hidden","true");
            document.getElementById("zoompage-zoomdisplay-tip-1b").removeAttribute("hidden");
            document.getElementById("zoompage-zoomdisplay-tip-2a").removeAttribute("hidden");
            document.getElementById("zoompage-zoomdisplay-tip-2b").setAttribute("hidden","true");
        }
        else
        {
            document.getElementById("zoompage-zoomdisplay-tip-1a").removeAttribute("hidden");
            document.getElementById("zoompage-zoomdisplay-tip-1b").setAttribute("hidden","true");
            document.getElementById("zoompage-zoomdisplay-tip-2a").setAttribute("hidden","true");
            document.getElementById("zoompage-zoomdisplay-tip-2b").removeAttribute("hidden");
        }
        
        if (zoomPage.imagePage(browser))  /* image page */
        {
            if (zoomPage.imagezoomMode == 0)
            {
                document.getElementById("zoompage-zoomdisplay-tip-3a").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-3b").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-3c").removeAttribute("hidden");
            }
            else
            {
                document.getElementById("zoompage-zoomdisplay-tip-3a").removeAttribute("hidden");
                document.getElementById("zoompage-zoomdisplay-tip-3b").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-3c").setAttribute("hidden","true");
            }
            
            document.getElementById("zoompage-zoomdisplay-tip-4a").setAttribute("hidden","true");
            document.getElementById("zoompage-zoomdisplay-tip-4b").setAttribute("hidden","true");
        }
        else  /* not image page */
        {
            if (zoomPage.zoomMode == 0 || zoomPage.zoomMode == 2)
            {
                document.getElementById("zoompage-zoomdisplay-tip-3a").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-3b").removeAttribute("hidden");
                document.getElementById("zoompage-zoomdisplay-tip-3c").setAttribute("hidden","true");
            }
            else
            {
                document.getElementById("zoompage-zoomdisplay-tip-3a").removeAttribute("hidden");
                document.getElementById("zoompage-zoomdisplay-tip-3b").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-3c").setAttribute("hidden","true");
            }
            
            if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full"))
            {
                
                document.getElementById("zoompage-zoomdisplay-tip-4a").setAttribute("hidden","true");
                document.getElementById("zoompage-zoomdisplay-tip-4b").removeAttribute("hidden");
            }
            else
            {
                document.getElementById("zoompage-zoomdisplay-tip-4a").removeAttribute("hidden");
                document.getElementById("zoompage-zoomdisplay-tip-4b").setAttribute("hidden","true");
            }
        }
    },
    
    onDisplayMouseDown: function(event)
    {
        var menuPopup,zoomDisplay;
        
        if (event.target.localName != "toolbarbutton") return;
        
        zoomPage.button = event.button;
        zoomPage.shiftKey = event.shiftKey;
        zoomPage.ctrlKey = event.ctrlKey;
        
        if (event.button == 1 && zoomPage.prefs.getBoolPref("swapmouseclicks"))  /* middle-click - clicks swapped */
        {
            zoomDisplay = document.getElementById("zoompage-zoomdisplay-" + zoomPage.activeButtonStyle());
            
            menuPopup = zoomDisplay.childNodes[0];
            
            menuPopup.openPopup(zoomDisplay,"after_start",0,0,false,false,null);
        }
    },
    
    onDisplayClick: function(event)
    {
        var browser;
        
        if (event.target.localName != "toolbarbutton") return;
        
        browser = gBrowser.selectedBrowser;
        
        if (event.button == 0 && !event.shiftKey && !event.ctrlKey && zoomPage.prefs.getBoolPref("swapmouseclicks") ||  /* click - mouse clicks swapped */
            event.button == 0 && event.ctrlKey && !zoomPage.prefs.getBoolPref("swapmouseclicks") ||  /* ctrl+click - mouse clicks not swapped */
            event.button == 1 && !zoomPage.prefs.getBoolPref("swapmouseclicks"))  /* middle-click - mouse clicks not swapped */
        {
            FullZoom.reset();
        }
        else if (event.button == 0 && event.shiftKey)  /* shift+click */
        {
            if (zoomPage.imagePage(browser))  /* image page */
            {
                if (zoomPage.imagezoomMode == 0) zoomPage.fittowindowZoom(browser,false);  /* image - fit-to-window zoom */
                else ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
            }
            else  /* not image page */
            {
                if (zoomPage.zoomMode == 0 || zoomPage.zoomMode == 2)
                {
                    zoomPage.fittowidthZoomAndAfter(true,browser,null,"odc",
                    function(browser,sitevalue)
                    {
                        if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                        else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                        else FullZoom._applySettingToPref();
                       
                        zoomPage.setPercent(31);
                    });
                }
                else
                {
                    zoomPage.defaultZoom(browser);
                    
                    if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                    else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                    else FullZoom._applySettingToPref();
                    
                    zoomPage.setPercent(32);
                }
            }
        }
        else if (event.button == 2)  /* right-click */
        {
            if (zoomPage.imagePage(browser)) ;  /* image page */
            else ZoomManager.toggleZoom();  /* not image page */
            
            event.preventDefault();
        }
    },
    
    onDisplayDblClick: function(event)
    {
        event.preventDefault();
    },
    
    onDisplayContextMenu: function(event)
    {
        event.preventDefault();
    },
    
    onZoomMenuPopupShowing: function(event)
    {
        var i,browser,menuPopup,menuItem,subMenuPopup,zoomDisplay,offsetX,offsetY;
        
        if (event.target.parentNode.localName != "toolbarbutton") return;
        
        menuPopup = event.target;
        
        browser = gBrowser.selectedBrowser;
        
        if (zoomPage.button == 0 && !zoomPage.shiftKey && !zoomPage.ctrlKey && !zoomPage.prefs.getBoolPref("swapmouseclicks") ||  /* click - mouse clicks not swapped */
            zoomPage.button == 0 && zoomPage.ctrlKey && zoomPage.prefs.getBoolPref("swapmouseclicks") ||  /* ctrl+click - mouse clicks swapped */
            zoomPage.button == 1 && zoomPage.prefs.getBoolPref("swapmouseclicks"))  /* middle-click - mouse clicks swapped */
        {
            for (i = menuPopup.childNodes.length-1; i >= 0; i--)
            {
                menuItem = menuPopup.childNodes[i];
                if (menuItem.getAttribute("value") == -2) menuItem.removeEventListener("popupshowing",zoomPage.onModeSubMenuPopupShowing,false);
                menuPopup.removeChild(menuItem);
            }
            
            for (i = 0; i < zoomPage.levelArray.length; i++)
            {
                menuItem = document.createElement("menuitem");
                menuPopup.appendChild(menuItem);
                menuItem.setAttribute("label",zoomPage.levelArray[i]);
                menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
                menuItem.setAttribute("value",zoomPage.levelArray[i]);
                
                if (zoomPage.prefs.getBoolPref("pcsuffix")) menuItem.setAttribute("label",menuItem.getAttribute("label") + "%");
                
                if (zoomPage.imagePage(browser))  /* image page */
                {
                    if (zoomPage.prefs.getIntPref("imgdefaultlevel") == zoomPage.levelArray[i])
                    {
                        menuItem.style.setProperty("color","#202020","");
                        menuItem.style.setProperty("font-weight","bold","");
                        menuItem.setAttribute("tooltip","zoompage-imgdefault-tip");
                    }
                }
                else  /* not image page */
                {
                    if (zoomPage.prefs.getIntPref("defaultlevel") == zoomPage.levelArray[i])
                    {
                        menuItem.style.setProperty("color","#202020","");
                        menuItem.style.setProperty("font-weight","bold","");
                        menuItem.setAttribute("tooltip","zoompage-default-tip");
                    }
                    if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full") && zoomPage.prefs.getIntPref("textminlevel") == zoomPage.levelArray[i])
                    {
                        menuItem.style.setProperty("color","#800000","");
                        menuItem.style.setProperty("font-weight","bold","");
                        menuItem.setAttribute("tooltip","zoompage-textmin-tip");
                    }
                }
            }
            
            menuItem = document.createElement("menuitem");
            menuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","menuitem-iconic zoompage-icon");
            menuItem.setAttribute("image","chrome://zoompage/skin/zoompage-menu-fittowidth.png");
            menuItem.setAttribute("value",-1);
            if (zoomPage.imagePage(browser)) menuItem.setAttribute("tooltip","zoompage-fittowindow-tip");
            else menuItem.setAttribute("tooltip","zoompage-fittowidth-tip");
            
            menuItem = document.createElement("separator");
            menuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","groove");
            menuItem.style.setProperty("margin-top","1px","");
            menuItem.style.setProperty("margin-bottom","0px","");
            
            menuItem = document.createElement("menu");
            menuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","menu-iconic zoompage-icon");
            menuItem.setAttribute("image","chrome://zoompage/skin/zoompage-menu-options.png");
            menuItem.setAttribute("value",-2);
            menuItem.setAttribute("tooltip","zoompage-options-tip");
            
            subMenuPopup = document.createElement("menupopup");
            menuItem.appendChild(subMenuPopup);
            subMenuPopup.setAttribute("position","end_after");
            subMenuPopup.addEventListener("popupshowing",zoomPage.onModeSubMenuPopupShowing,false);
            
            zoomDisplay = document.getElementById("zoompage-zoomdisplay-" + zoomPage.activeButtonStyle());
            
            offsetX = (zoomDisplay.boxObject.width-menuPopup.boxObject.width)/2;
            if (zoomPage.osPlatform == "WINNT") { if (zoomPage.activeButtonStyle() == "s") offsetX += 2; }
            else if (zoomPage.osPlatform == "Linux") offsetX += 0;
            else if (zoomPage.osPlatform == "Darwin") offsetX += 1;
            
            if (zoomPage.osPlatform == "WINNT") { if (zoomPage.activeButtonStyle() == "t") offsetY = 1; else offsetY = -3; }
            else if (zoomPage.osPlatform == "Linux") offsetY = 0;
            else if (zoomPage.osPlatform == "Darwin")  { if (zoomPage.activeButtonStyle() == "t") offsetY = 2; else offsetY = 1; }
            
            menuPopup.moveTo(menuPopup.boxObject.screenX+offsetX,menuPopup.boxObject.screenY+offsetY);
        }
        else event.preventDefault();
    },
    
    onModeSubMenuPopupShowing: function(event)
    {
        var i,browser,subMenuPopup,menuItem,offsetX,offsetY;
        
        subMenuPopup = event.target;
        
        browser = gBrowser.selectedBrowser;
        
        for (i = subMenuPopup.childNodes.length-1; i >= 0; i--)
        {
            menuItem = subMenuPopup.childNodes[i];
            subMenuPopup.removeChild(menuItem);
        }
        
        if (zoomPage.imagePage(browser))  /* image page */
        {
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-1dflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-22);
            if (zoomPage.prefs.getIntPref("imagedefftwtype") == 0)
            {
                menuItem.style.setProperty("color","#202020","");
                menuItem.style.setProperty("font-weight","bold","");
                menuItem.setAttribute("tooltip","zoompage-imagedefftwtype-tip");
            }
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-2dflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-23);
            if (zoomPage.prefs.getIntPref("imagedefftwtype") == 1)
            {
                menuItem.style.setProperty("color","#202020","");
                menuItem.style.setProperty("font-weight","bold","");
                menuItem.setAttribute("tooltip","zoompage-imagedefftwtype-tip");
            }
            
            menuItem = document.createElement("separator");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","groove");
            menuItem.style.setProperty("margin-top","1px","");
            menuItem.style.setProperty("margin-bottom","0px","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-defflag").value + "/" + document.getElementById("zoompage-expflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-20);
            menuItem.setAttribute("tooltip","zoompage-defaultzoom-tip");
            if (zoomPage.prefs.getIntPref("imagezoommode") == 0) menuItem.style.setProperty("font-weight","bold","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-ftwflag").value + "/" + document.getElementById("zoompage-expflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-21);
            menuItem.setAttribute("tooltip","zoompage-fittowindow-tip");
            if (zoomPage.prefs.getIntPref("imagezoommode") == 1) menuItem.style.setProperty("font-weight","bold","");
        }
        else  /* not image page */
        {
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-toflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-14);
            if (zoomPage.prefs.getIntPref("defaulttype") == 0)
            {
                menuItem.style.setProperty("color","#202020","");
                menuItem.style.setProperty("font-weight","bold","");
                menuItem.setAttribute("tooltip","zoompage-defaulttype-tip");
            }
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-fpflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-15);
            if (zoomPage.prefs.getIntPref("defaulttype") == 1)
            {
                menuItem.style.setProperty("color","#202020","");
                menuItem.style.setProperty("font-weight","bold","");
                menuItem.setAttribute("tooltip","zoompage-defaulttype-tip");
            }
            
            menuItem = document.createElement("separator");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("class","groove");
            menuItem.style.setProperty("margin-top","1px","");
            menuItem.style.setProperty("margin-bottom","0px","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-defflag").value + "/" + document.getElementById("zoompage-expflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-10);
            menuItem.setAttribute("tooltip","zoompage-defaultzoom-tip");
            if (zoomPage.prefs.getIntPref("zoommode") == 0) menuItem.style.setProperty("font-weight","bold","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-ftwflag").value + "/" + document.getElementById("zoompage-expflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-11);
            menuItem.setAttribute("tooltip","zoompage-fittowidth-tip");
            if (zoomPage.prefs.getIntPref("zoommode") == 1) menuItem.style.setProperty("font-weight","bold","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-defflag").value + "/" + document.getElementById("zoompage-ssflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-12);
            menuItem.setAttribute("tooltip","zoompage-defzoomsitespec-tip");
            if (zoomPage.prefs.getIntPref("zoommode") == 2) menuItem.style.setProperty("font-weight","bold","");
            
            menuItem = document.createElement("menuitem");
            subMenuPopup.appendChild(menuItem);
            menuItem.setAttribute("label",document.getElementById("zoompage-ftwflag").value + "/" + document.getElementById("zoompage-ssflag").value);
            menuItem.setAttribute("class",menuItem.getAttribute("class") + " zoompage-text");
            menuItem.setAttribute("value",-13);
            menuItem.setAttribute("tooltip","zoompage-ftwzoomsitespec-tip");
            if (zoomPage.prefs.getIntPref("zoommode") == 3) menuItem.style.setProperty("font-weight","bold","");
        }
        
        if (zoomPage.osPlatform == "WINNT") { offsetX = 3; offsetY = -3; }
        else if (zoomPage.osPlatform == "Linux")  { offsetX = 0; offsetY = -2; }
        else if (zoomPage.osPlatform == "Darwin")  { offsetX = 2; offsetY = -4; }
        
        subMenuPopup.style.setProperty("margin-left",offsetX + "px","");
        subMenuPopup.style.setProperty("margin-bottom",offsetY + "px","");
    },
    
    onMenuItemClick: function(event)
    {
        var level,browser,optionsWindow,zoomDisplay,menuPopup;
        
        level = event.target.getAttribute("value");
        
        if (event.button == 0 || event.button == 1)  /* click or middle-click */
        {
            
            if (level >= 0)  /* percentage value */
            {
                browser = gBrowser.selectedBrowser;
                
                if (event.ctrlKey || event.button == 1)  /* ctrl+click or middle-click */
                {
                    /* Set Default Zoom or Image Default Zoom level */
                    
                    if (zoomPage.imagePage(browser))  /* image page */
                    {
                        zoomPage.ignorePrefChanged = true;
                        
                        zoomPage.prefs.setIntPref("imgdefaultlevel",level);  /* set image default zoom level */
                    }
                    else  /* not image page */
                    {
                        zoomPage.ignorePrefChanged = true;
                        
                        zoomPage.prefs.setIntPref("defaultlevel",level);  /* set default zoom level */
                        if (zoomPage.zoomMode == 0 || zoomPage.zoomMode == 2) zoomPage.setGlobalSiteSpecific(FullZoom.name,level/100);  /* set global zoom value content preference */
                    }
                }
                else if (event.shiftKey)  /* shift+click */
                {
                    /* Set Text Minimum Zoom level */
                    
                    if (zoomPage.imagePage(browser)) ;  /* image page */
                    else  /* not image page */
                    {
                        zoomPage.ignorePrefChanged = true;
                        
                        zoomPage.prefs.setIntPref("textminlevel",level);  /* set text minimum zoom level */
                        
                        ZoomManager.zoom = ZoomManager.zoom;
                    }
                }
                else
                {
                    /* Set zoom value */
                    
                    ZoomManager.zoom = level/100;  /* set current zoom */
                    
                    if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                    else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                    else FullZoom._applySettingToPref();
                }
                
                zoomPage.setPercent(33);
            }
            else if (level == -1)  /* fit-to-width/window */
            {
                browser = gBrowser.selectedBrowser;
                
                if (zoomPage.imagePage(browser))  /* image page */
                {
                    zoomPage.fittowindowZoom(browser,(event.ctrlKey || event.button == 1));  /* image - fit-to-window zoom */
                    
                    zoomPage.setPercent(34);
                }
                else  /* not image page */
                {
                    zoomPage.fittowidthZoomAndAfter(true,browser,null,"omic",
                    function(browser,sitevalue)
                    {
                        if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                        else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                        else FullZoom._applySettingToPref();
                        
                        zoomPage.setPercent(35);
                    });
                }
            }
            else if (level == -2) /* options */
            {
                zoomDisplay = document.getElementById("zoompage-zoomdisplay-" + zoomPage.activeButtonStyle());
                
                menuPopup = zoomDisplay.childNodes[0];
                
                menuPopup.hidePopup();
                
                optionsWindow = zoomPage.winmed.getMostRecentWindow("zoompage-options");
                
                if (optionsWindow) optionsWindow.focus();
                else window.openDialog("chrome://zoompage/content/zoompage-options.xul","","chrome,dialog,titlebar,centerscreen",null);
            }
            else if (level <= -10 && level >= -13) /* zoom modes */
            {
                zoomPage.prefs.setIntPref("zoommode",-level-10);  /* set zoom mode */
            }
            else if (level <= -14 && level >= -15) /* zoom types */
            {
                if (event.ctrlKey || event.button == 1)  /* ctrl+click or middle-click */
                {
                    /* Set Default Zoom Type */
                    
                    zoomPage.ignorePrefChanged = true;
                    
                    zoomPage.prefs.setIntPref("defaulttype",-level-14);  /* set default zoom type */
                }
                else
                {
                    /* Set zoom type */
                    
                    if (ZoomManager.useFullZoom != -level-14) ZoomManager.toggleZoom();
                }
            }
            else if (level <= -20 && level >= -21) /* image zoom modes */
            {
                zoomPage.prefs.setIntPref("imagezoommode",-level-20);  /* set image zoom mode */
            }
            else if (level <= -22 && level >= -23) /* fit-to-window types */
            {
                if (event.ctrlKey || event.button == 1)  /* ctrl+click or middle-click */
                {
                    /* Set Default Fit-to-Window Type */
                    
                    zoomPage.ignorePrefChanged = true;
                    
                    zoomPage.prefs.setIntPref("imagedefftwtype",-level-22);  /* set default fit-to-window type */
                }
                else
                {
                    /* Set fit-to-window type */
                    
                    browser = gBrowser.selectedBrowser;
                    
                    browser.zoompage_ftwType = -level-22;
                    
                    zoomPage.fittowindowZoom(browser,false);
                }
            }
            
            if (event.button == 1)
            {
                zoomDisplay = document.getElementById("zoompage-zoomdisplay-" + zoomPage.activeButtonStyle());
                
                menuPopup = zoomDisplay.childNodes[0];
                
                menuPopup.hidePopup();
            }
        
            // if (zoomPage.ffVersion >= +"29.0") CustomizableUI.hidePanelForNode(document.getElementById("zoompage-item"));
        }
    },
    
    /********************************************************************/
    
    /* Event handlers for tab and page zoom initiation events */
    
    onTabOpen: function(event)
    {
        var tab,browser;
        
        tab = event.target;
        
        browser = tab.linkedBrowser;
        
        /* Initialize page state */
        
        browser.zoompage_pageState = 0;  /* no page */
        
        /* Initialize normal/image zoom values*/
        
        browser.zoompage_normalZoom = zoomPage.prefs.getIntPref("defaultlevel")/100;
        browser.zoompage_imageZoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
        
        /* Initialize not zoomed flag */
        
        browser.zoompage_notZoomed = true;
        
        /* Initialize resize timeout */
        
        browser.zoompage_resizeTimeout = null;
        
        /* Initialize previous uri and previous width/height */
        
        browser.zoompage_previousURI = "";
        browser.zoompage_previousWidth = browser.boxObject.width;
        browser.zoompage_previousHeight = browser.boxObject.height;
        
        /* Initialize last fit-to-width/window zoom values */
        
        browser.zoompage_lastFitToWidth = zoomPage.lastFitToWidth;
        browser.zoompage_lastFitToWindow = 1.0;
        
        /* Initialize and remember zoom type for tab */
        
        ZoomManager.useFullZoom = zoomPage.prefs.getIntPref("defaulttype");
        browser.zoompage_zoomType = ZoomManager.useFullZoom;
        
        /* Initialize and remember fit-to-window type for tab */
        
        browser.zoompage_ftwType = zoomPage.prefs.getIntPref("imagedefftwtype");
        
        /* Default Zoom - set zoom value for tab opened - page not loaded yet */
        
        if (zoomPage.zoomMode == 0)
        {
            zoomPage.defaultZoom(browser);
            
            zoomPage.setPercent(41);
        }
    },
    
    onDOMTitleChanged: function(message)
    {
        var browser;
        
        /* Event happens once or twice when image is first visible and again just before pageshow event */
        /* Image width and height are now available - but zooming here can cause partial image display */
        /* Frame script sets opacity to 0.999 here and 1.000 on pageshow event to force image repaint */
        
        browser = message.target;
        
        if (zoomPage.imagePage(browser))  /* image page */
        {
            /* Set image width and height */
            
            browser.zoompage_imageWidth = message.json.width;
            browser.zoompage_imageHeight = message.json.height;
            
            /* Default Zoom - set zoom value for page loaded */
            
            if (zoomPage.imagezoomMode == 0)
            {
                ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("imgdefaultlevel")/100);
            }
            
            /* Fit-To-Window Zoom - set zoom value for page loaded */
            
            if (zoomPage.imagezoomMode == 1)
            {
                zoomPage.fittowindowZoom(browser,false);  /* image - fit-to-window zoom */
            }
            
            zoomPage.setPercent(42);
        }
    },

    onDOMContentLoaded: function(message)
    {
        var browser;
        
        browser = message.target;
        
        if (zoomPage.errorPage(browser))  /* error page */
        {
            if (zoomPage.prefs.getBoolPref("specialenable")) ZoomManager.setZoomForBrowser(browser,1.0);
            else ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("defaultlevel")/100);
            
            browser.zoompage_errorShown = true;
        }
    },
    
    onPageShow: function(message)
    {
        var browser;
        
        browser = message.target;
        
        if (browser.getAttribute("src").substr(0,9) == "resource:" ||
            browser.getAttribute("src").substr(0,14) == "moz-extension:") return;  /* browser inside extension button popup panel */
        
        /* Update page state */
        
        browser.zoompage_pageState = (message.json.state ? 2 : 1);  /* page loaded or page loading */
        
        if (zoomPage.errorPage(browser))  /* error page */
        {
            if (zoomPage.prefs.getBoolPref("specialenable")) ZoomManager.setZoomForBrowser(browser,1.0);
            else ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("defaultlevel")/100);
            
            browser.zoompage_errorShown = true;
        }
        else if (zoomPage.imagePage(browser))  /* image page */
        {
            /* Default Zoom - set zoom value for page loaded */
            
            if (zoomPage.imagezoomMode == 0)
            {
                if (!browser.zoompage_imageShown)
                    ZoomManager.setZoomForBrowser(browser,zoomPage.prefs.getIntPref("imgdefaultlevel")/100);
            }
            
            /* Fit-To-Window Zoom - set zoom value for page loaded */
            
            if (zoomPage.imagezoomMode == 1)
            {
                if (!browser.zoompage_imageShown)
                    zoomPage.fittowindowZoom(browser,false);  /* image - fit-to-window zoom */
            }
            
            browser.zoompage_previousURI = browser.currentURI.spec;
            
            browser.zoompage_imageShown = true;
            
            zoomPage.setPercent(43);
        }
        else  /* not image page */
        {
            zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
            function(browser,sitevalue)
            {
                var doftw;
                
                doftw = false;
                
                /* Site-Specific Zoom - set zoom value for page loaded or reloaded - already applied by onLocationChange() */
                
                /* Special Pages - about:blank treated as special page only if page loaded */
                
                /* Default Zoom - set zoom value for different page loaded */
                
                if (zoomPage.zoomMode == 0)
                {
                    if (zoomPage.specialPage(browser))  /* special page - set special zoom */
                    {
                        ZoomManager.setZoomForBrowser(browser,zoomPage.defaultLevel(browser)/100);
                    }
                    else if (browser.zoompage_imageShown || browser.zoompage_specialShown || browser.zoompage_errorShown)  /* leaving image/special/error page - reinstate normal zoom */
                    {
                        ZoomManager.setZoomForBrowser(browser,browser.zoompage_normalZoom);
                    }
                    else if (browser.zoompage_previousURI == "") zoomPage.defaultZoom(browser);  /* first page show for tab */
                }
                
                /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for different page loaded */
                
                if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                {
                    if (browser.currentURI.spec != browser.zoompage_previousURI)  /* ignore minor refreshes/updates which don't change uri */
                    {
                        doftw = true;  /* about:blank is treated as special page only if page loaded */
                    }
                }
                
                zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"ops",
                function(browser,sitevalue)
                {
                    /* Synchronize Firefox's Site-Specific Zoom global zoom value content preference */
                    
                    if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                    {
                        zoomPage.setGlobalSiteSpecific(FullZoom.name,browser.zoompage_lastFitToWidth);  /* set global zoom value content preference same as fit-to-width zoom */
                    }
                    else
                    {
                        zoomPage.setGlobalSiteSpecific(FullZoom.name,zoomPage.defaultLevel(browser)/100);  /* set global zoom value content preference same as default zoom */
                    }
                    
                    browser.zoompage_previousURI = browser.currentURI.spec;
                    
                    if (zoomPage.specialPage(browser)) browser.zoompage_specialShown = true;
                    
                    zoomPage.setPercent(44);
                });
            });
        }
	if ( !browser.currentURI ) {
		return
	}
        else if (browser.currentURI.spec.indexOf("#") != -1) {
		zoomPage.onHashChange(message);
		}
    },

    onHashChange: function(message)
    {
        var browser;
        
        browser = message.target;
        
        window.setTimeout(
        function(browser)
        {
            if (zoomPage.imagePage(browser))  /* image page */
            {
                browser.zoompage_previousURI = browser.currentURI.spec;
                
                zoomPage.setPercent(45);
            }
            else  /* not image page */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    var doftw;
                    
                    doftw = false;
                    
                    /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for URI hash changed */
                    
                    if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                    {
                        doftw = true;
                    }
                    
                    zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"ohc",
                    function(browser,sitevalue)
                    {
                        /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom - update global content preference  */
                        
                        if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                        {
                            zoomPage.setGlobalSiteSpecific(FullZoom.name,browser.zoompage_lastFitToWidth);  /* set global zoom value content preference same as fit-to-width zoom */
                        }
                        
                        browser.zoompage_previousURI = browser.currentURI.spec;
                        
                        zoomPage.setPercent(46);
                    });
                });
            }
        }
        ,200,browser);  /* allow time for contents to update after URI hash changed */
    },

    onTabSelect: function(event)
    {
        var browser;
        
        browser = gBrowser.selectedBrowser;
        
        if (zoomPage.imagePage(browser))  /* image page */
        {
            /* Adjust zoom value for selected tab - in case zoom levels have been changed */
            
            if (ZoomManager.zoom < 1/100) ZoomManager.zoom = 1/100;
            else if (ZoomManager.zoom > 10000/100) ZoomManager.zoom = 10000/100;
            
            /* Fit-To-Window Zoom - in case zoom mode or browser size has been changed */
            
            if (zoomPage.imagezoomMode == 1)
            {
                if (browser.boxObject.width != browser.zoompage_previousWidth || browser.boxObject.height != browser.zoompage_previousHeight)
                {
                    zoomPage.fittowindowZoom(browser,false);
                }
            }
            
            browser.zoompage_previousWidth = browser.boxObject.width;
            browser.zoompage_previousHeight = browser.boxObject.height;
            
            zoomPage.setPercent(47);
        }
        else  /* not image page */
        {
            if (browser.zoompage_pageState >= 1)  /* page loading or page loaded */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    var doftw;
                    
                    doftw = false;
                    
                    /* Site-Specific Zoom - set zoom value for page loaded or reloaded - already applied by onLocationChange() */
                    
                    /* Adjust zoom value for selected tab - in case zoom levels have been changed */
                    
                    if (ZoomManager.zoom < ZoomManager.MIN) ZoomManager.zoom = ZoomManager.MIN;
                    else if (ZoomManager.zoom > ZoomManager.MAX) ZoomManager.zoom = ZoomManager.MAX;
                    
                    /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - in case zoom mode has been changed */
                    
                    if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                    {
                        if (browser.boxObject.width != browser.zoompage_previousWidth)
                        {
                            doftw = true;
                        }
                    }
                    
                    zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"ots",
                    function(browser,sitevalue)
                    {
                        /* Site-Specific Zoom - update saved site-specific value */
                        
                        if (zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3)
                        {
                            if (typeof sitevalue != "undefined")
                            {
                                if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                                else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                                else FullZoom._applySettingToPref();
                            }
                        }
                        
                        /* Synchronize Firefox's Site-Specific Zoom global content preference */
                        
                        if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                        {
                            zoomPage.setGlobalSiteSpecific(FullZoom.name,browser.zoompage_lastFitToWidth);  /* set global zoom value content preference same as fit-to-width zoom */
                        }
                        else
                        {
                            zoomPage.setGlobalSiteSpecific(FullZoom.name,zoomPage.defaultLevel(browser)/100);  /* set global zoom value content preference same as default zoom */
                        }
                        
                        browser.zoompage_previousWidth = browser.boxObject.width;
                        browser.zoompage_previousHeight = browser.boxObject.height;
                        
                        zoomPage.setPercent(48);
                    });
                });
            }
        }
    },
    
    /********************************************************************/
    
    /* Event handlers for browser resize/scroll, content click, and reload events */
    
    onBrowserResize: function(message)
    {
        var browser,timeout;
        
        browser = message.target;
        
        if (browser.getAttribute("src").substr(0,9) == "resource:" ||
            browser.getAttribute("src").substr(0,14) == "moz-extension:") return;  /* browser inside extension button popup panel */
        
        if (document.getElementById("main-window").hasAttribute("tiletabs-ffversion") ||
            document.getElementById("main-window").hasAttribute("tileview-ffversion"))  /* Tile Tabs or Tile View add-ons installed */
        {
            if (gBrowser.getNotificationBox(browser).style.getPropertyValue("visibility") == "hidden") return;
        }
        else  /* Tile Tabs and Tile View add-ons not installed */
        {
            if (browser != gBrowser.selectedBrowser) return;
        }
        
        // if (browser.zoompage_resizeTimeout != null) return;  /* to update zoom during resize */
        if (browser.zoompage_resizeTimeout != null) window.clearTimeout(browser.zoompage_resizeTimeout);  /* to update zoom at end of resize */
        
        if (zoomPage.imagePage(browser)) timeout = 10; else timeout = 100;
        
        browser.zoompage_resizeTimeout = window.setTimeout(
        function(browser)
        {
            var width,height;
            
            if (zoomPage.imagePage(browser))  /* image page */
            {
                width = browser.boxObject.width;
                height = browser.boxObject.height;
                
                /* Fit-To-Window Zoom - set zoom value for browser window resized */
                
                if (zoomPage.imagezoomMode == 1)
                {
                    if (width != browser.zoompage_previousWidth || height != browser.zoompage_previousHeight)  /* ignore resizes that don't change browser window size */
                    {
                        zoomPage.fittowindowZoom(browser,false);  /* image - fit-to-window zoom */
                    }
                }
                
                browser.zoompage_previousWidth = browser.boxObject.width;
                browser.zoompage_previousHeight = browser.boxObject.height;
                browser.zoompage_resizeTimeout = null;
                
                zoomPage.setImageBackground(browser,true);  /* Default Zoom - resize does not zoom image so have to update background here */
                
                zoomPage.setImageCursor(browser);  /* Default Zoom - resize does not zoom image so have to update cursor here */
                
                zoomPage.setPercent(51);
            }
            else  /* not image page */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    var doftw,width,height;
                    
                    doftw = false;
                    
                    width = browser.boxObject.width;
                    height = browser.boxObject.height;
                    
                    /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for browser window resized */
                    
                    if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                    {
                        if (width != browser.zoompage_previousWidth)  /* ignore resizes that don't change browser window width */
                        {
                            doftw = true;
                        }
                    }
                    
                    zoomPage.fittowidthZoomAndAfter(doftw,browser,sitevalue,"obr",
                    function(browser,sitevalue)
                    {
                        /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom - update global content preference  */
                        
                        if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                        {
                            if (browser == gBrowser.selectedBrowser) zoomPage.setGlobalSiteSpecific(FullZoom.name,browser.zoompage_lastFitToWidth);  /* set global zoom value content preference same as fit-to-width zoom */
                        }
                        
                        browser.zoompage_previousWidth = browser.boxObject.width;
                        browser.zoompage_previousHeight = browser.boxObject.height;
                        browser.zoompage_resizeTimeout = null;
                        
                        if (doftw) zoomPage.setPercent(52);
                    });
                });
            }
        }
        ,timeout,browser);
    },
    
    onBrowserScroll: function(message)
    {
        if (message.json.detail < 0) FullZoom.enlarge(); else FullZoom.reduce();
        
        zoomPage.setPercent(53);
    },
    
    onContentClick: function(message)  /* image page */
    {
        var browser,clientX,clientY,scrollX,scrollY,scrollMaxX,scrollMaxY;
        var imageWidth,imageHeight,imageCentre,imageMargin,browserWidth,browserHeight,cursorX,cursorY;
        
        browser = message.target;
        
        if (!browser.zoompage_imageShown) return;
        
        if (!zoomPage.prefs.getBoolPref("imgclickftwdef")) return;
        
        /* Viewing image - alternate clicks on image restore to initial size or fit-to-window */
        /* Apply fit-to-window or default zoom and calculate cursor coordinates */
        
        clientX = message.json.clientx;
        clientY = message.json.clienty;
        
        scrollX = message.json.scrollx;
        scrollY = message.json.scrolly;
        
        scrollMaxX = message.json.scrollmaxx;
        scrollMaxY = message.json.scrollmaxy;
        
        imageWidth = browser.zoompage_imageWidth*ZoomManager.getZoomForBrowser(browser);
        imageHeight = browser.zoompage_imageHeight*ZoomManager.getZoomForBrowser(browser);
        
        if ((zoomPage.prefs.getIntPref("imageposition") & 0x02) == 2) imageCentre = true; else imageCentre = false;
        if ((zoomPage.prefs.getIntPref("imageposition") & 0x01) == 1) imageMargin = 10; else imageMargin = 0;
        
        if (scrollMaxY > 0) browserWidth = browser.boxObject.width-17;
        else browserWidth = browser.boxObject.width;
        
        if (scrollMaxX > 0) browserHeight = browser.boxObject.height-17;
        else browserHeight = browser.boxObject.height;
        
        if (imageCentre && scrollMaxX == 0) cursorX = clientX-(browserWidth-imageWidth)/2;
        else cursorX = clientX-imageMargin+scrollX;
        
        if (imageCentre && scrollMaxY == 0) cursorY = clientY-(browserHeight-imageHeight)/2;
        else cursorY = clientY-imageMargin+scrollY;
        
        cursorX = cursorX/ZoomManager.zoom;
        cursorY = cursorY/ZoomManager.zoom;
        
        if (zoomPage.imagezoomMode == 0 && Math.round(ZoomManager.zoom*100) == zoomPage.prefs.getIntPref("imgdefaultlevel") ||
            zoomPage.imagezoomMode == 1 && Math.round(ZoomManager.zoom*100) != Math.round(browser.zoompage_lastFitToWindow*100))
        {
            zoomPage.fittowindowZoom(browser,message.json.ctrlkey);  /* image page - fit-to-window zoom */
        }
        else ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;  /* image page - default zoom */
        
        cursorX = cursorX*ZoomManager.zoom;
        cursorY = cursorY*ZoomManager.zoom;
        
        browser.messageManager.sendAsyncMessage("zoompage-onclick-reply",
            { cursorx: cursorX, cursory: cursorY, browserwidth: browser.boxObject.width, browserheight: browser.boxObject.height },{ });
        
        zoomPage.setPercent(54);
    },
    
    onReloadClick: function(event)
    {
        var browser;
        
        browser = gBrowser.selectedBrowser;
        
        zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
        function(browser,sitevalue)
        {
            /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for page manually reloaded */
            
            if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
            {
                browser.zoompage_previousURI = "";
            }
        });
    },
    
    onReloadAllClick: function(event)
    {
        var i,browser;
        
        for (i = 0; i < gBrowser.tabContainer.childNodes.length; i++)
        {
            browser = gBrowser.tabContainer.childNodes[i].linkedBrowser;
            
            zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
            function(browser,sitevalue)
            {
                /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for page manually reloaded */
                
                if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
                {
                    browser.zoompage_previousURI = "";
                }
            });
        }
    },
    
    /********************************************************************/
    
    /* Event handlers for keyboard shortcut events */
    
    onWindowKeyPress: function(event)
    {
        var browser;
        
        /* Event fires three times in e10s window - unless event.stopPropagation() is called */
        
        browser = gBrowser.selectedBrowser;
        
        /* Intercept Firefox shortcuts for page manually reloaded */ 
        
        zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
        function(browser,sitevalue)
        {
            /* Fit-To-Width Zoom or Fit-To-Width Zoom + Site-Specific Zoom (if site-specific not set) - set zoom value for page manually reloaded */
            
            if (zoomPage.zoomMode == 1 || (zoomPage.zoomMode == 3 && typeof sitevalue == "undefined"))
            {
                if ((event.keyCode == event.DOM_VK_F5))
                {
                    browser.zoompage_previousURI = "";
                }
                else if ((event.charCode == "r".charCodeAt(0) || event.charCode == "R".charCodeAt(0)) && (event.ctrlKey || event.metaKey))
                {
                    browser.zoompage_previousURI = "";
                }
            }
        });
        
        /* Ignore space characters - space is pref value if a configurable shortcut is disabled */
        
        if (event.charCode == " ".charCodeAt(0)) return;
        
        /* Configurable Shortcut - Default or Fit-To-Width/Window zoom mode */
        
        if (event.charCode == zoomPage.prefs.getCharPref("zmshortdefftw").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser))  /* image page */
            {
                zoomPage.prefs.setIntPref("imagezoommode",zoomPage.prefs.getIntPref("imagezoommode") ^ 0x01);
            }
            else  /* not image page */
            {
                zoomPage.prefs.setIntPref("zoommode",zoomPage.prefs.getIntPref("zoommode") ^ 0x01);
            }
            
            zoomPage.showStatusPanel(0,null);      
            
            zoomPage.setPercent(63);
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Site-Specific or Explicit zoom mode */
        
        if (event.charCode == zoomPage.prefs.getCharPref("zmshortssexp").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser)) ;  /* image page */
            else  /* not image page */
            {
                zoomPage.prefs.setIntPref("zoommode",zoomPage.prefs.getIntPref("zoommode") ^ 0x02);
                
                zoomPage.showStatusPanel(0,null);      
            }
            
            zoomPage.setPercent(64);
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Fit-To-Width or Default zoom level */
        
        if (event.charCode == zoomPage.prefs.getCharPref("zlshortftwdef").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser))  /* image page */
            {
                if (zoomPage.imagezoomMode == 0)
                {
                    if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWindow*100) &&
                        Math.round(browser.zoompage_lastFitToWindow*100) != zoomPage.prefs.getIntPref("imgdefaultlevel"))  /* avoid getting stuck at default zoom level */
                    {
                        ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
                    }
                    else zoomPage.fittowindowZoom(browser,false);  /* image page - fit-to-window zoom */
                }
                else
                {
                    if (Math.round(ZoomManager.zoom*100) == zoomPage.prefs.getIntPref("imgdefaultlevel"))
                    {
                        zoomPage.fittowindowZoom(browser,false);  /* image page - fit-to-window zoom */
                    }
                    else ZoomManager.zoom = zoomPage.prefs.getIntPref("imgdefaultlevel")/100;
                }
                
                zoomPage.showStatusPanel(1,null);      
                
                zoomPage.setPercent(65);
            }
            else  /* not image page */
            {
                if (zoomPage.zoomMode == 0 || zoomPage.zoomMode == 2)
                {
                    if (Math.round(ZoomManager.zoom*100) == Math.round(browser.zoompage_lastFitToWidth*100) &&
                        Math.round(browser.zoompage_lastFitToWidth*100) != zoomPage.defaultLevel(browser))  /* avoid getting stuck at default zoom level */
                    {
                        zoomPage.defaultZoom(browser);
                        
                        if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                        else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                        else FullZoom._applySettingToPref();
                        
                        zoomPage.showStatusPanel(2,null);      
                        
                        zoomPage.setPercent(66);
                    }
                    else
                    {
                        zoomPage.fittowidthZoomAndAfter(true,browser,null,"owkpd",
                        function(browser,sitevalue)
                        {
                            if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                            else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                            else FullZoom._applySettingToPref();
                            
                            zoomPage.showStatusPanel(2,null);      
                            
                            zoomPage.setPercent(67);
                        });
                    }
                }
                else if (zoomPage.zoomMode == 1 || zoomPage.zoomMode == 3)
                {
                    if (Math.round(ZoomManager.zoom*100) == zoomPage.defaultLevel(browser))
                    {
                        zoomPage.fittowidthZoomAndAfter(true,browser,null,"owkpf",
                        function(browser,sitevalue)
                        {
                            if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                            else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                            else FullZoom._applySettingToPref();
                            
                            zoomPage.showStatusPanel(2,null);      
                            
                            zoomPage.setPercent(68);
                        });
                    }
                    else
                    {
                        zoomPage.defaultZoom(browser);
                        
                        if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
                        else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
                        else FullZoom._applySettingToPref();
                        
                        zoomPage.showStatusPanel(2,null);      
                        
                        zoomPage.setPercent(69);
                    }
                }
            }
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Ignore Site-Specific zoom level */
        
        if (event.charCode == zoomPage.prefs.getCharPref("zlshortignoress").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser)) ;  /* image page */
            else  /* not image page */
            {
                zoomPage.getDomainSiteSpecific(browser,FullZoom.name,
                function(browser,sitevalue)
                {
                    if (zoomPage.zoomMode == 2 || zoomPage.zoomMode == 3)
                    {
                        if (typeof sitevalue != "undefined")
                        {
                            if (Math.round(ZoomManager.zoom*100) == Math.round(sitevalue*100))
                            {
                                if (zoomPage.zoomMode == 2)
                                {
                                    zoomPage.defaultZoom(browser);
                                    
                                    zoomPage.setPercent(71);
                                }
                                else
                                {
                                    zoomPage.fittowidthZoomAndAfter(true,browser,sitevalue,"owkps",
                                    function(browser,sitevalue)
                                    {
                                        zoomPage.setPercent(72);
                                    });
                                }
                            }
                            else
                            {
                                ZoomManager.zoom = sitevalue;
                                
                                zoomPage.setPercent(73);
                            }
                        }
                        
                        zoomPage.showStatusPanel(3,sitevalue);      
                    }
                });
            }
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Full-Page or Text-Only zoom */
        
        if (event.charCode == zoomPage.prefs.getCharPref("ztshortfpto").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser)) ;  /* image page */
            else  /* not image page */
            {
                ZoomManager.toggleZoom();
                
                zoomPage.showStatusPanel(4,null);
            }
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - 1D or 2D Fit-To-Width zoom */
        
        if (event.charCode == zoomPage.prefs.getCharPref("ztshort1d2d").charCodeAt(0) && (event.ctrlKey || event.metaKey))
        {
            if (zoomPage.imagePage(browser))  /* image page */
            {
                browser.zoompage_ftwType = browser.zoompage_ftwType ^ 0x01;
                
                zoomPage.fittowindowZoom(browser,false);  /* image page - fit-to-window zoom */
                
                zoomPage.showStatusPanel(5,null);
            }
            else ;  /* not image page */
            
            zoomPage.setPercent(74);
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Zoom In by zoom step (or to next level) */
        
        if ((event.charCode == zoomPage.prefs.getCharPref("zsshortzoomin").charCodeAt(0) && (event.ctrlKey || event.metaKey) && event.shiftKey))
        {
            ZoomManager.enlarge(true);
            
            if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
            else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
            else FullZoom._applySettingToPref();
            
            zoomPage.setPercent(61);
            
            event.preventDefault();
            event.stopPropagation();
        }
        
        /* Configurable Shortcut - Zoom Out by zoom step (or to next level) */
        
        if ((event.charCode == zoomPage.prefs.getCharPref("zsshortzoomout").charCodeAt(0) && (event.ctrlKey || event.metaKey) && event.shiftKey))
        {
            ZoomManager.reduce(true);
            
            if (zoomPage.ffVersion >= +"24.0") FullZoom._applyZoomToPref(browser);
            else if (zoomPage.ffVersion >= +"22.0") FullZoom._applySettingToPref(false,null);
            else FullZoom._applySettingToPref();
            
            zoomPage.setPercent(62);
            
            event.preventDefault();
            event.stopPropagation();
        }
    },
    
    /********************************************************************/
    
    /* Event handlers for toolbar button customization events */
    
    onButtonDrop: function(event)  /* Firefox 4.0-28.0 */
    {
        zoomPage.setPercent(81);
        
        if (document.getElementById("reload-button") != null) document.getElementById("reload-button").addEventListener("click",zoomPage.onReloadClick,false);
    },

    onPanelUIPopupShowing: function(event)  /* Firefox 29.0+ */
    {
        zoomPage.setPercent(82);
    },
    
    onOverflowPopupShowing: function(event)  /* Firefox 29.0+ */
    {
        zoomPage.setPercent(83);
    },
    
    /********************************************************************/
    
    /* Debug functions */
    
    debugMessage: function(module,information)
    {
        var info;
        
        zoomPage.debugCount++;
        info = document.getElementById("zoompage-debug-info");
        info.textContent = zoomPage.debugCount + " - " + module + " - " + information;
    },
    
    debugMessageFrameScript: function(message)
    {
        zoomPage.debugMessage(message.json.module,message.json.information);
    }
};

/************************************************************************/

/* Start Zoom Page */

window.addEventListener("load",zoomPage.onLoad,false);
window.addEventListener("unload",zoomPage.onUnload,false);

/************************************************************************/
