/************************************************************************/
/*                                                                      */
/*      Zoom Page  -  Firefox Extension  -  Options Window              */
/*                                                                      */
/*      Javascript for Options dialog                                   */
/*                                                                      */
/*      Copyright (C) 2009-2017  by  DW-dev                             */
/*                                                                      */
/*      Last Edit  -  27 Feb 2017                                       */
/*                                                                      */
/************************************************************************/

"use strict";

var zoomPageOptions =
{
    appInfo: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
    versionComparator: Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator),
    ffVersion: "",
    
    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.zoompage."),
    
    promptService: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),
    
    zoomLevels: new Array("30-50-67-80-90-100-110-120-133-150-170-200-240-300",
                          "30-50-75-100-125-150-200-300-400",
                          "30-50-75-90-100-110-125-150-200-300-400",
                          "30-50-67-80-90-100-110-120-133-150-170-200-240-300-400",
                          "25-50-75-100-125-150-200-300-400-500",
                          "25-50-75-90-100-110-125-150-200-300-400-500",
                          "25-50-67-80-90-100-110-120-133-150-170-200-240-300-400-500"),
    
    initPrefs: function()
    {
        /* Determine Firefox version and set attribute */
        
        if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"34.0a1") >= 0) zoomPageOptions.ffVersion = "34.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"30.0a1") >= 0) zoomPageOptions.ffVersion = "30.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"29.0a1") >= 0) zoomPageOptions.ffVersion = "29.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"27.0a1") >= 0) zoomPageOptions.ffVersion = "27.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"25.0a1") >= 0) zoomPageOptions.ffVersion = "25.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"24.0a1") >= 0) zoomPageOptions.ffVersion = "24.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"22.0a1") >= 0) zoomPageOptions.ffVersion = "22.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"19.0a1") >= 0) zoomPageOptions.ffVersion = "19.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"17.0a1") >= 0) zoomPageOptions.ffVersion = "17.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"14.0a1") >= 0) zoomPageOptions.ffVersion = "14.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"12.0a1") >= 0) zoomPageOptions.ffVersion = "12.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"11.0a1") >= 0) zoomPageOptions.ffVersion = "11.0";
        else if (zoomPageOptions.versionComparator.compare(zoomPageOptions.appInfo.version,"9.0a1") >= 0) zoomPageOptions.ffVersion = "9.0";
        else zoomPageOptions.ffVersion = "4.0";
        
        document.getElementById("zoompage-options").setAttribute("zoompage-ffversion",zoomPageOptions.ffVersion);
        
        /* Set user preferences branch */
        
        zoomPageOptions.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.zoompage.");
        
        /* Load user preferences */
        
        zoomPageOptions.loadPrefs();
    },
    
    loadPrefs: function()
    {
        var i,menulist,checkbox,textbox,tooltip,radiogroup;
        var defaultlevel,textminlevel,fittowidthmin,fittowidthmax,fittowidthful,imgdefaultlevel,imgfittowindowmin,imgfittowindowmax;
        var levelArray = new Array();
        
        /* Zoom Mode */
        
        menulist = document.getElementById("zoompage-zoommode");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("zoommode");
        }
        catch(e)
        {
            menulist.selectedIndex = 2;
        }
        
        zoomPageOptions.onChangeZoomMode();
        
        /* Default Zoom Type */
        
        menulist = document.getElementById("zoompage-defaulttype");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("defaulttype");
        }
        catch(e)
        {
            menulist.selectedIndex = 1;
        }
        
        /* Image Zoom Mode */
        
        menulist = document.getElementById("zoompage-imagezoommode");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("imagezoommode");
        }
        catch(e)
        {
            menulist.selectedIndex = 1;
        }
        
        zoomPageOptions.onChangeImageZoomMode();
        
        /* Default Fit-To-Window Type */
        
        menulist = document.getElementById("zoompage-imagedefftwtype");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("imagedefftwtype");
        }
        catch(e)
        {
            menulist.selectedIndex = 1;
        }
        
        /* Button Style */
        
        menulist = document.getElementById("zoompage-buttonstyle");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("buttonstyle");
        }
        catch(e)
        {
            menulist.value = "n";
        }
        
        /* Button Colors */
        
        menulist = document.getElementById("zoompage-buttoncolors");
        try
        {
            menulist.value = zoomPageOptions.prefs.getIntPref("buttoncolors");
        }
        catch(e)
        {
            menulist.value = 0;
        }
        
        /* Swap Mouse Clicks */
        
        checkbox = document.getElementById("zoompage-swapmouseclicks");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("swapmouseclicks");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Use Zoom Step */
        
        checkbox = document.getElementById("zoompage-usezoomstep");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("usezoomstep");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Show Firefox Zoom Indicator */
        
        checkbox = document.getElementById("zoompage-showindicator");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("showindicator");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Explicit Zoom - X Prefix */
        
        checkbox = document.getElementById("zoompage-expprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("expprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Default Zoom - D Prefix */
        
        checkbox = document.getElementById("zoompage-defprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("defprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Default Zoom - Dotted Border */
        
        checkbox = document.getElementById("zoompage-defborder");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("defborder");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Fit-To-Width Zoom - F Prefix */
        
        checkbox = document.getElementById("zoompage-ftwprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("ftwprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Fit-To-Width Zoom - Dashed Border */
        
        checkbox = document.getElementById("zoompage-ftwborder");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("ftwborder");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Site-Specific Zoom - S Prefix */
        
        checkbox = document.getElementById("zoompage-ssprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("ssprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Site-Specific Zoom - Solid Border */
        
        checkbox = document.getElementById("zoompage-ssborder");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("ssborder");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Full-Page - P Prefix */
        
        checkbox = document.getElementById("zoompage-fpprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("fpprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Text-Only - T Prefix */
        
        checkbox = document.getElementById("zoompage-toprefix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("toprefix");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Text-Only - Italic Display */
        
        checkbox = document.getElementById("zoompage-toitalic");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("toitalic");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Text-Only - Red Display */
        
        checkbox = document.getElementById("zoompage-tored");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("tored");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Percent Sign  - % Suffix */
        
        checkbox = document.getElementById("zoompage-pcsuffix");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("pcsuffix");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Custom Zoom Levels */
        
        try
        {
            zoomPageOptions.zoomLevels[7] = zoomPageOptions.prefs.getCharPref("customlevels");
        }
        catch(e)
        {
            zoomPageOptions.zoomLevels[7] = "100";
        }
        
        /* Current Zoom Levels */
        
        menulist = document.getElementById("zoompage-zoomlevels");
        menulist.removeAllItems();
        menulist.appendItem(document.getElementById("zoompage-firefoxmarker").value + "  " + zoomPageOptions.zoomLevels[0]);
        menulist.appendItem("1:  " + zoomPageOptions.zoomLevels[1]);
        menulist.appendItem("2:  " + zoomPageOptions.zoomLevels[2]);
        menulist.appendItem("3:  " + zoomPageOptions.zoomLevels[3]);
        menulist.appendItem("4:  " + zoomPageOptions.zoomLevels[4]);
        menulist.appendItem("5:  " + zoomPageOptions.zoomLevels[5]);
        menulist.appendItem("6:  " + zoomPageOptions.zoomLevels[6]);
        menulist.appendItem(document.getElementById("zoompage-custommarker").value + "  " + zoomPageOptions.zoomLevels[7]);
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("zoomlevels");
        }
        catch(e)
        {
            menulist.selectedIndex = 0;
        }
        
        tooltip = document.getElementById("zoompage-zoomlevels-tooltip-label");
        tooltip.setAttribute("value",menulist.getItemAtIndex(menulist.selectedIndex).label.substr(4));
        
        /* Zoom Step */
        
        textbox = document.getElementById("zoompage-zoomstep");
        try
        {
            textbox.value = zoomPageOptions.prefs.getIntPref("zoomstep");
        }
        catch(e)
        {
            textbox.value = "1";
        }
        
        /* Edit Custom Zoom Levels */
        
        textbox = document.getElementById("zoompage-editcustomlevels");
        try
        {
            textbox.value = zoomPageOptions.prefs.getCharPref("editcustomlevels");
        }
        catch(e)
        {
            textbox.value = "100";
        }
        
        /* Default Zoom Level */
        
        levelArray = zoomPageOptions.zoomLevels[document.getElementById("zoompage-zoomlevels").selectedIndex].split("-");
        
        menulist = document.getElementById("zoompage-defaultlevel");
        try
        {
            defaultlevel = zoomPageOptions.prefs.getIntPref("defaultlevel");
        }
        catch(e)
        {
            defaultlevel = 100;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (levelArray[i] == defaultlevel) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = 100;
        defaultlevel = menulist.value;
        
        /* Text Minimum Zoom Level */
        
        menulist = document.getElementById("zoompage-textminlevel");
        try
        {
            textminlevel = zoomPageOptions.prefs.getIntPref("textminlevel");
        }
        catch(e)
        {
            textminlevel = 30;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (levelArray[i] == textminlevel) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = levelArray[levelArray.length-1];
        
        /* Fit-To-Width Zoom Minimum */
        
        menulist = document.getElementById("zoompage-fittowidthmin");
        try
        {
            fittowidthmin = zoomPageOptions.prefs.getIntPref("fittowidthmin");
        }
        catch(e)
        {
            fittowidthmin = 67;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= defaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= defaultlevel && levelArray[i] == fittowidthmin) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = defaultlevel;
        
        /* Fit-To-Width Zoom Maximum (half-screen window) */
        
        menulist = document.getElementById("zoompage-fittowidthmax");
        try
        {
            fittowidthmax = zoomPageOptions.prefs.getIntPref("fittowidthmax");
        }
        catch(e)
        {
            fittowidthmax = 120;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= defaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= defaultlevel && levelArray[i] == fittowidthmax) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = defaultlevel;
        fittowidthmax = menulist.value;
        
        /* Fit-To-Width Zoom Maximum (full-screen window) */
        
        menulist = document.getElementById("zoompage-fittowidthful");
        try
        {
            fittowidthful = zoomPageOptions.prefs.getIntPref("fittowidthful");
        }
        catch(e)
        {
            fittowidthful = 150;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= fittowidthmax) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= fittowidthmax && levelArray[i] == fittowidthful) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = fittowidthmax;
        
        /* Special Pages Zoom Level */
        
        checkbox = document.getElementById("zoompage-specialenable");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("specialenable");
        }
        catch(e)
        {
            checkbox.checked = false;
        }
        
        /* Image Default Zoom Level */
        
        levelArray = zoomPageOptions.zoomLevels[document.getElementById("zoompage-zoomlevels").selectedIndex].split("-");
        
        menulist = document.getElementById("zoompage-imgdefaultlevel");
        try
        {
            imgdefaultlevel = zoomPageOptions.prefs.getIntPref("imgdefaultlevel");
        }
        catch(e)
        {
            imgdefaultlevel = 100;
        }
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        for (i = 0; i < levelArray.length; i++) if (levelArray[i] == imgdefaultlevel) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = 100;
        imgdefaultlevel = menulist.value;
        
        /* Image Fit-To-Window Zoom Minimum */
        
        menulist = document.getElementById("zoompage-imgfittowindowmin");
        try
        {
            imgfittowindowmin = zoomPageOptions.prefs.getIntPref("imgfittowindowmin");
        }
        catch(e)
        {
            imgfittowindowmin = 1;
        }
        levelArray.unshift("10");
        levelArray.unshift("1");
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= imgdefaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.children[0].children[1].label = "[10]";
        menulist.children[0].children[0].label = "[1]";
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= imgdefaultlevel && levelArray[i] == imgfittowindowmin) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = imgdefaultlevel;
        levelArray.shift();
        levelArray.shift();
        
        /* Image Fit-To-Window Zoom Maximum */
        
        menulist = document.getElementById("zoompage-imgfittowindowmax");
        try
        {
            imgfittowindowmax = zoomPageOptions.prefs.getIntPref("imgfittowindowmax");
        }
        catch(e)
        {
            imgfittowindowmax = 300;
        }
        levelArray.push("1000");
        levelArray.push("10000");
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= imgdefaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.children[0].children[menulist.children[0].children.length-2].label = "[1000]";
        menulist.children[0].children[menulist.children[0].children.length-1].label = "[10000]";
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= imgdefaultlevel && levelArray[i] == imgfittowindowmax) break;
        if (i < levelArray.length) menulist.value = levelArray[i]; else menulist.value = imgdefaultlevel;
        levelArray.pop();
        levelArray.pop();
        
        /* Image Position */
        
        menulist = document.getElementById("zoompage-imageposition");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("imageposition");
        }
        catch(e)
        {
            menulist.selectedIndex = 2;
        }
        
        /* Image Background */
        
        menulist = document.getElementById("zoompage-imagebackground");
        try
        {
            menulist.selectedIndex = zoomPageOptions.prefs.getIntPref("imagebackground");
        }
        catch(e)
        {
            menulist.selectedIndex = 1;
        }
        
        zoomPageOptions.onChangeImageBackground();
        
        /* Image Around Color */
        
        textbox = document.getElementById("zoompage-imagearoundcolor");
        try
        {
            textbox.value = zoomPageOptions.prefs.getCharPref("imagearoundcolor");
        }
        catch(e)
        {
            textbox.value = "#222222";
        }
        
        /* Image Behind Color */
        
        textbox = document.getElementById("zoompage-imagebehindcolor");
        try
        {
            textbox.value = zoomPageOptions.prefs.getCharPref("imagebehindcolor");
        }
        catch(e)
        {
            textbox.value = "#E5E5E5";
        }
        
        /* Click on image to apply Fit-To-Width or Default zoom level */
        
        checkbox = document.getElementById("zoompage-imgclickftwdef");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("imgclickftwdef");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Horizontal scroll with mouse wheel (if no vertical scroll bar) */
        
        checkbox = document.getElementById("zoompage-imghorizscroll");
        try
        {
            checkbox.checked = zoomPageOptions.prefs.getBoolPref("imghorizscroll");
        }
        catch(e)
        {
            checkbox.checked = true;
        }
        
        /* Shortcut - Default or Fit-To-Width zoom mode */
        
        menulist = document.getElementById("zoompage-zmshortdefftw");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zmshortdefftw");
        }
        catch(e)
        {
            menulist.value = "[";
        }
        
        /* Shortcut - Site-Specific or Explicit zoom mode */
        
        menulist = document.getElementById("zoompage-zmshortssexp");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zmshortssexp");
        }
        catch(e)
        {
            menulist.value = "]";
        }
        
        /* Shortcut - Fit-To-Width or Default zoom level */
        
        menulist = document.getElementById("zoompage-zlshortftwdef");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zlshortftwdef");
        }
        catch(e)
        {
            menulist.value = "'";
        }
        
        /* Shortcut - Ignore Site-Specific zoom level */
        
        menulist = document.getElementById("zoompage-zlshortignoress");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zlshortignoress");
        }
        catch(e)
        {
            menulist.value = "\\";
        }
        
        /* Shortcut - Full-Page or Text-Only zoom */
        
        menulist = document.getElementById("zoompage-ztshortfpto");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("ztshortfpto");
        }
        catch(e)
        {
            menulist.value = ";";
        }
        
        /* Shortcut - 1D or 2D Fit-To-Window zoom */
        
        menulist = document.getElementById("zoompage-ztshort1d2d");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("ztshort1d2d");
        }
        catch(e)
        {
            menulist.value = ";";
        }
        
        /* Shortcut - Zoom In by zoom step (or to next level) */
        
        menulist = document.getElementById("zoompage-zsshortzoomin");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zsshortzoomin");
        }
        catch(e)
        {
            menulist.value = "+";
        }
        
        /* Shortcut - Zoom Out by zoom step (or to next level) */
        
        menulist = document.getElementById("zoompage-zsshortzoomout");
        try
        {
            menulist.value = zoomPageOptions.prefs.getCharPref("zsshortzoomout");
        }
        catch(e)
        {
            menulist.value = "_";
        }
    },
    
    onChangeZoomMode: function()
    {
        var menulist;
        
        menulist = document.getElementById("zoompage-zoommode");
        menulist.setAttribute("tooltiptext",menulist.children[0].children[menulist.selectedIndex].getAttribute("tooltiptext"));
    },
    
    onChangeImageZoomMode: function()
    {
        var menulist;
        
        menulist = document.getElementById("zoompage-imagezoommode");
        menulist.setAttribute("tooltiptext",menulist.children[0].children[menulist.selectedIndex].getAttribute("tooltiptext"));
    },
    
    onChangeImageBackground: function()
    {
        var menulist,label,textbox;
        
        menulist = document.getElementById("zoompage-imagebackground");
        
        label = document.getElementById("zoompage-imagearoundlabel");
        textbox = document.getElementById("zoompage-imagearoundcolor");
        
        label.disabled = textbox.disabled = (menulist.selectedIndex <= 1);
        
        label = document.getElementById("zoompage-imagebehindlabel");
        textbox = document.getElementById("zoompage-imagebehindcolor");
        
        label.disabled = textbox.disabled = (menulist.selectedIndex <= 1);
    },
    
    onChangeZoomLevels: function()
    {
        var i,menulist,tooltip;
        var defaultlevel,textminlevel,fittowidthmin,fittowidthmax,fittowidthful,imgdefaultlevel,imgfittowindowmin,imgfittowindowmax;
        var levelArray = new Array();
        
        function snapToLevel(level)
        {
            for (i = 0; i < levelArray.length; i++)
                if (Number(levelArray[i]) >= level)
                {
                    if (i > 0 && level-Number(levelArray[i-1]) < Number(levelArray[i])-level) i--;
                    return Number(levelArray[i]);
                }
                
            return Number(levelArray[i-1]);
        }
        
        menulist = document.getElementById("zoompage-zoomlevels");
        tooltip = document.getElementById("zoompage-zoomlevels-tooltip-label");
        tooltip.setAttribute("value",menulist.getItemAtIndex(menulist.selectedIndex).label.substr(4));
        
        levelArray = zoomPageOptions.zoomLevels[document.getElementById("zoompage-zoomlevels").selectedIndex].split("-");
        
        menulist = document.getElementById("zoompage-defaultlevel");
        defaultlevel = snapToLevel(menulist.value);
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = defaultlevel;
        
        menulist = document.getElementById("zoompage-textminlevel");
        textminlevel = snapToLevel(menulist.value);
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = textminlevel;
        
        menulist = document.getElementById("zoompage-fittowidthmin");
        fittowidthmin = snapToLevel(menulist.value);
        if (fittowidthmin > defaultlevel) fittowidthmin = defaultlevel;
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= defaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = fittowidthmin;
        
        menulist = document.getElementById("zoompage-fittowidthmax");
        fittowidthmax = snapToLevel(menulist.value);
        if (fittowidthmax < defaultlevel) fittowidthmax = defaultlevel;
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= defaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = fittowidthmax;
        
        menulist = document.getElementById("zoompage-fittowidthful");
        fittowidthful = snapToLevel(menulist.value);
        if (fittowidthful < fittowidthmax) fittowidthful = fittowidthmax;
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= fittowidthmax) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = fittowidthful;
        
        menulist = document.getElementById("zoompage-imgdefaultlevel");
        imgdefaultlevel = snapToLevel(menulist.value);
        menulist.removeAllItems();
        for (i = 0; i < levelArray.length; i++) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.value = imgdefaultlevel;
        
        menulist = document.getElementById("zoompage-imgfittowindowmin");
        imgfittowindowmin = snapToLevel(menulist.value);
        if (imgfittowindowmin > imgdefaultlevel) imgfittowindowmin = imgdefaultlevel;
        menulist.removeAllItems();
        levelArray.unshift("10");
        levelArray.unshift("1");
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) <= imgdefaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.children[0].children[1].label = "[10]";
        menulist.children[0].children[0].label = "[1]";
        menulist.value = imgfittowindowmin;
        levelArray.shift();
        levelArray.shift();
        
        menulist = document.getElementById("zoompage-imgfittowindowmax");
        imgfittowindowmax = snapToLevel(menulist.value);
        if (imgfittowindowmax < imgdefaultlevel) imgfittowindowmax = imgdefaultlevel;
        menulist.removeAllItems();
        levelArray.push("1000");
        levelArray.push("10000");
        for (i = 0; i < levelArray.length; i++) if (Number(levelArray[i]) >= imgdefaultlevel) menulist.appendItem(levelArray[i],levelArray[i]);
        menulist.children[0].children[menulist.children[0].children.length-2].label = "[1000]";
        menulist.children[0].children[menulist.children[0].children.length-1].label = "[10000]";
        menulist.value = imgfittowindowmax;
        levelArray.pop();
        levelArray.pop();
    },
    
    onCommandCustomCopy: function()
    {
        var menulist,textbox;
        
        /* Copy current zoom levels to custom zoom levels */
        
        menulist = document.getElementById("zoompage-zoomlevels");
        
        textbox = document.getElementById("zoompage-editcustomlevels");
        
        textbox.value = menulist.label.substr(4);
    },
    
    onCommandCustomValidate: function()
    {
        var i,textbox,textstr,levels;
        var levelArray = new Array();
        
        /* Validate and re-order custom zoom levels */
        
        textbox = document.getElementById("zoompage-editcustomlevels");
        
        textstr = textbox.value;
        
        for (i = 0; i < textstr.length; i++)
        {
            if (textstr.charAt(i) >= "0" && textstr.charAt(i) <= "9") continue;
            if (textstr.charAt(i) == "-") continue;
            textstr = textstr.substr(0,i) + textstr.substr(i+1);
            i--;
        }
        
        levelArray = textstr.split("-");
        levelArray.push("100");
        levelArray.sort(function(a,b){ return a-b; });
        
        for (i = 0; i < levelArray.length; i++)
        {
            if (Number(levelArray[i]) < 1 || Number(levelArray[i]) > 10000) { levelArray.splice(i,1); i--; }
            if (i > 0 && levelArray[i] == levelArray[i-1]) { levelArray.splice(i,1); i--; }
        }
        
        levels = "" + levelArray[0];
        for (i = 1; i < levelArray.length; i++) levels = levels + "-" + levelArray[i];
        
        textbox.value = levels;
    },
    
    onCommandCustomUpdate: function()
    {
        var textbox,menulist,index;
        
        /* Update custom entry in current zoom levels menu list */
        
        zoomPageOptions.onCommandCustomValidate();
        
        textbox = document.getElementById("zoompage-editcustomlevels");
        
        zoomPageOptions.zoomLevels[7] = textbox.value;
        
        menulist = document.getElementById("zoompage-zoomlevels");
        index = menulist.selectedIndex;
        
        menulist.removeItemAt(7);
        menulist.appendItem(document.getElementById("zoompage-custommarker").value + "  " + zoomPageOptions.zoomLevels[7]);
        
        menulist.selectedIndex = index;
        
        if (index == 7) zoomPageOptions.onChangeZoomLevels();
    },
    
    savePrefs: function()
    {
        var i;
        
        /* Set user preferences branch */
        
        zoomPageOptions.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.zoompage.");
        
        /* Zoom Mode */
        
        zoomPageOptions.prefs.setIntPref("zoommode",document.getElementById("zoompage-zoommode").selectedIndex);
        
        /* Default Zoom Type */
        
        zoomPageOptions.prefs.setIntPref("defaulttype",document.getElementById("zoompage-defaulttype").selectedIndex);
        
        /* Image Zoom Mode */
        
        zoomPageOptions.prefs.setIntPref("imagezoommode",document.getElementById("zoompage-imagezoommode").selectedIndex);
        
        /* Default Fit-To-Window Type */
        
        zoomPageOptions.prefs.setIntPref("imagedefftwtype",document.getElementById("zoompage-imagedefftwtype").selectedIndex);
        
        /* Button Style */
        
        zoomPageOptions.prefs.setCharPref("buttonstyle",document.getElementById("zoompage-buttonstyle").value);
        
        /* Button Colors */
        
        zoomPageOptions.prefs.setIntPref("buttoncolors",document.getElementById("zoompage-buttoncolors").value);
        
        /* Swap Mouse Clicks */
        
        zoomPageOptions.prefs.setBoolPref("swapmouseclicks",document.getElementById("zoompage-swapmouseclicks").checked);
        
        /* Use Zoom Step */
        
        zoomPageOptions.prefs.setBoolPref("usezoomstep",document.getElementById("zoompage-usezoomstep").checked);
        
        /* Show Firefox Zoom Indicator */
        
        zoomPageOptions.prefs.setBoolPref("showindicator",document.getElementById("zoompage-showindicator").checked);
        
        /* Explicit Zoom - X Prefix */
        
        zoomPageOptions.prefs.setBoolPref("expprefix",document.getElementById("zoompage-expprefix").checked);
        
        /* Default Zoom - D Prefix */
        
        zoomPageOptions.prefs.setBoolPref("defprefix",document.getElementById("zoompage-defprefix").checked);
        
        /* Default Zoom - Dotted Border */
        
        zoomPageOptions.prefs.setBoolPref("defborder",document.getElementById("zoompage-defborder").checked);
        
        /* Fit-To-Width Zoom - F Prefix */
        
        zoomPageOptions.prefs.setBoolPref("ftwprefix",document.getElementById("zoompage-ftwprefix").checked);
        
        /* Fit-To-Width Zoom - Dashed Border */
        
        zoomPageOptions.prefs.setBoolPref("ftwborder",document.getElementById("zoompage-ftwborder").checked);
        
        /* Site-Specific Zoom - S Prefix */
        
        zoomPageOptions.prefs.setBoolPref("ssprefix",document.getElementById("zoompage-ssprefix").checked);
        
        /* Site-Specific Zoom - Solid Border */
        
        zoomPageOptions.prefs.setBoolPref("ssborder",document.getElementById("zoompage-ssborder").checked);
        
        /* Full-Page - P Prefix */
        
        zoomPageOptions.prefs.setBoolPref("fpprefix",document.getElementById("zoompage-fpprefix").checked);
        
        /* Text-Only - T Prefix */
        
        zoomPageOptions.prefs.setBoolPref("toprefix",document.getElementById("zoompage-toprefix").checked);
        
        /* Text-Only - Italic Display */
        
        zoomPageOptions.prefs.setBoolPref("toitalic",document.getElementById("zoompage-toitalic").checked);
        
        /* Text-Only - Red Display */
        
        zoomPageOptions.prefs.setBoolPref("tored",document.getElementById("zoompage-tored").checked);
        
        /* Percent Sign - % Suffix */
        
        zoomPageOptions.prefs.setBoolPref("pcsuffix",document.getElementById("zoompage-pcsuffix").checked);
        
        /* Current Zoom Levels */
        
        zoomPageOptions.prefs.setIntPref("zoomlevels",document.getElementById("zoompage-zoomlevels").selectedIndex);
        
        /* Zoom Step */
        
        zoomPageOptions.prefs.setIntPref("zoomstep",document.getElementById("zoompage-zoomstep").value);
        
        /* Custom Zoom Levels */
        
        zoomPageOptions.prefs.setCharPref("customlevels",zoomPageOptions.zoomLevels[7]);
        
        /* Edit Custom Zoom Levels */
        
        zoomPageOptions.prefs.setCharPref("editcustomlevels",document.getElementById("zoompage-editcustomlevels").value);
        
        /* Default Zoom Level */
        
        zoomPageOptions.prefs.setIntPref("defaultlevel",document.getElementById("zoompage-defaultlevel").value);
        
        /* Text Minimum Zoom Level */
        
        zoomPageOptions.prefs.setIntPref("textminlevel",document.getElementById("zoompage-textminlevel").value);
        
        /* Fit-To-Width Zoom Minimum */
        
        zoomPageOptions.prefs.setIntPref("fittowidthmin",document.getElementById("zoompage-fittowidthmin").value);
        
        /* Fit-To-Width Zoom Maximum (half-screen window) */
        
        zoomPageOptions.prefs.setIntPref("fittowidthmax",document.getElementById("zoompage-fittowidthmax").value);
        
        /* Fit-To-Width Zoom Maximum (full-screen window) */
        
        zoomPageOptions.prefs.setIntPref("fittowidthful",document.getElementById("zoompage-fittowidthful").value);
        
        /* Special Pages Zoom Level */
        
        zoomPageOptions.prefs.setBoolPref("specialenable",document.getElementById("zoompage-specialenable").checked);
        
        /* Image Default Zoom Level */
        
        zoomPageOptions.prefs.setIntPref("imgdefaultlevel",document.getElementById("zoompage-imgdefaultlevel").value);
        
        /* Fit-To-Width Zoom Minimum */
        
        zoomPageOptions.prefs.setIntPref("imgfittowindowmin",document.getElementById("zoompage-imgfittowindowmin").value);
        
        /* Fit-To-Width Zoom Maximum */
        
        zoomPageOptions.prefs.setIntPref("imgfittowindowmax",document.getElementById("zoompage-imgfittowindowmax").value);
        
        /* Image Position */
        
        zoomPageOptions.prefs.setIntPref("imageposition",document.getElementById("zoompage-imageposition").selectedIndex);
        
        /* Image Background */
        
        zoomPageOptions.prefs.setIntPref("imagebackground",document.getElementById("zoompage-imagebackground").selectedIndex);
        
        /* Image Around Color */
        
        zoomPageOptions.prefs.setCharPref("imagearoundcolor",document.getElementById("zoompage-imagearoundcolor").value);
        
        /* Image Behind Color */
        
        zoomPageOptions.prefs.setCharPref("imagebehindcolor",document.getElementById("zoompage-imagebehindcolor").value);
        
        /* Click on image to apply Fit-To-Width or Default zoom level */
        
        zoomPageOptions.prefs.setBoolPref("imgclickftwdef",document.getElementById("zoompage-imgclickftwdef").checked);
        
        /* Horizontal scroll with mouse wheel (if no vertical scroll bar) */
        
        zoomPageOptions.prefs.setBoolPref("imghorizscroll",document.getElementById("zoompage-imghorizscroll").checked);
        
        /* Shortcut - Default or Fit-To-Width zoom mode */
        
        zoomPageOptions.prefs.setCharPref("zmshortdefftw",document.getElementById("zoompage-zmshortdefftw").value);
        
        /* Shortcut - Site-Specific or Explicit zoom mode */
        
        zoomPageOptions.prefs.setCharPref("zmshortssexp",document.getElementById("zoompage-zmshortssexp").value);
        
        /* Shortcut - Fit-To-Width or Default zoom level */
        
        zoomPageOptions.prefs.setCharPref("zlshortftwdef",document.getElementById("zoompage-zlshortftwdef").value);
        
        /* Shortcut - Ignore Site-Specific zoom level */
        
        zoomPageOptions.prefs.setCharPref("zlshortignoress",document.getElementById("zoompage-zlshortignoress").value);
        
        /* Shortcut - Full-Page or Text-Only zoom */
        
        zoomPageOptions.prefs.setCharPref("ztshortfpto",document.getElementById("zoompage-ztshortfpto").value);
        
        /* Shortcut - 1D or 2D Fit-To-Window zoom */
        
        zoomPageOptions.prefs.setCharPref("ztshort1d2d",document.getElementById("zoompage-ztshort1d2d").value);
        
        /* Shortcut - Zoom In by zoom step (or to next level) */
        
        zoomPageOptions.prefs.setCharPref("zsshortzoomin",document.getElementById("zoompage-zsshortzoomin").value);
        
        /* Shortcut - Zoom Out by zoom step (or to next level) */
        
        zoomPageOptions.prefs.setCharPref("zsshortzoomout",document.getElementById("zoompage-zsshortzoomout").value);
    },
    
    restoreDefaults: function()
    {
        var title,message;
        
        /* Confirm restore defaults */
        
        title = document.getElementById("zoompage-prompttitle").getAttribute("value");
        message = document.getElementById("zoompage-options-confirmdefaults").getAttribute("value");
        if (!zoomPageOptions.promptService.confirm(null,title,message)) return;
        
        /* Set default preferences branch */
        
        zoomPageOptions.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getDefaultBranch("extensions.zoompage.");
        
        /* Load default preferences */
        
        zoomPageOptions.loadPrefs();
    }
};
